import 'package:example/Screen/SignUpScreen.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:google_sign_in/google_sign_in.dart';

FirebaseAuth auth = FirebaseAuth.instance;
final GoogleSignIn googleSignIn = GoogleSignIn();

String name;
String email;
String imageUrl;
String token;
String uid;
bool emailVerified;
String bearToken;

Future<String> signInWithGoogle() async {
  await Firebase.initializeApp();

  final GoogleSignInAccount googleSignInAccount = await googleSignIn.signIn();
  final GoogleSignInAuthentication googleSignInAuthentication =
  await googleSignInAccount.authentication;

  final AuthCredential credential = GoogleAuthProvider.credential(
    accessToken: googleSignInAuthentication.accessToken,
    idToken: googleSignInAuthentication.idToken,
  );

  final UserCredential authResult = await auth.signInWithCredential(credential);
  final User user = authResult.user;

  if (user != null) {
    // Checking if email and name is null
    assert(user.email != null);
    assert(user.displayName != null);
    assert(user.refreshToken != null);
    assert(user.photoURL != null);
    assert(user.uid != null);
    assert(user.emailVerified != null);

    auth.currentUser.getIdToken().then((value) => bearToken = value);

    name = user.displayName;
    email = user.email;
    imageUrl = user.photoURL;
    token = user.refreshToken;
    uid = user.uid;
    emailVerified = user.emailVerified;

    // Only taking the first part of the name, i.e., First Name
    if (name.contains(" ")) {
      name = name.substring(0, name.indexOf(" "));
    }

    assert(!user.isAnonymous);
    assert(await user.getIdToken() != null);

    final User currentUser = auth.currentUser;
    assert(user.uid == currentUser.uid);

    return '$user';
  }

  return null;
}

Future<void> signOutGoogle(BuildContext context) async {
  await googleSignIn.signOut().then((value) async {
    SharedPreferences preferences = await SharedPreferences.getInstance();
    await preferences.clear();

    Navigator.pushAndRemoveUntil(
        context,
        PageRouteBuilder(pageBuilder: (BuildContext context,
            Animation animation, Animation secondaryAnimation) {
          return SignUpScreen();
        }, transitionsBuilder: (BuildContext context,
            Animation<double> animation,
            Animation<double> secondaryAnimation,
            Widget child) {
          return new SlideTransition(
            position: new Tween<Offset>(
              begin: const Offset(1.0, 0.0),
              end: Offset.zero,
            ).animate(animation),
            child: child,
          );
        }),
            (Route route) => false);
  }).catchError((onError) {

  });
}

/*Future <FirebaseUser> signUp(email, password) async {
  try {
    FirebaseUser user = await auth.createUserWithEmailAndPassword(email: email, password: password);
    assert(user != null);
    assert(await user.getIdToken() != null);
    return user;
  } catch (e) {
    return null;
  }
}*/
