import 'dart:convert';
import 'package:example/Screen/ViewMoreScreen.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/DataModalComponent/SlidableListView.dart';
import 'package:bui_component/DrawerComonent/kfDrawer.dart';
import 'package:bui_component/HomeScreenComponent.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/FontSize.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Screen/ButtonView.dart';
import 'package:example/Screen/CardStyle.dart';
import 'package:example/Screen/DemoScreen.dart';
import 'package:example/Screen/ThemePickerScreen.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';

import '../Utils/class_builder.dart';

class HomeScreen extends KFDrawerContent {
  final Function() onMenuIconPressed;

  HomeScreen({this.onMenuIconPressed});

  @override
  _HomeScreenState createState() => _HomeScreenState();
}

class _HomeScreenState extends State<HomeScreen> {
  _HomeScreenState();

  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;

  AccountDetailsBean accountDetailsBean;

  List<AccountListBean> accountListBeanData = new List();

  bool isLoader = true;

  SharedPreferences prefs;

  @override
  void initState() {
    ClassBuilder.registerClasses();
    callPostAPI();
    getStringValuesSF();
    super.initState();
  }

  List<String> actions = ["Edit", "Delete"];

  List<PopupMenuItem> popUpMenu = new List();

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }

    popUpMenu = [
      PopupMenuItem(
        height: 30,
        value: 1,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.settings,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.SETTINGS,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 2,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.notifications_active,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.NOTIFICATION,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 3,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.messenger_rounded,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.MESSAGE,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 4,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.color_lens_sharp,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.THEME,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 5,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.feedback_rounded,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.FEED_BACK,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
    ];
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: HomeScreenComponent(
            onMenuPressed: onMenuPressed,
            backgroundColor: theme,
            onMessageFunction: onMessageFunction,
            onNotificationFunction: onNotificationFunction,
            onSettingFunction: onSettingFunction,
            onPopUpMenuFunction: onPopUpMenuPressed,
            popUpMenu: popUpMenu,
            child: isLoader
                ? Stack(
                    children: [
                      deleteDetailsView,
                      //  addDetailsView,
                      isLoaderView,
                    ],
                  )
                : Stack(
                    children: [
                      // appBar,
                      deleteDetailsView,
                      //addDetailsView,
                      detailsView
                    ],
                  )),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        margin: EdgeInsets.only(top: 90),
        child: ListView.builder(
          itemCount: accountListBeanData.length ?? 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                          //lineHeight: 10,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      margin: EdgeInsets.only(top: 60),
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        "Swipe left to use the edit and delete option",
        style: TextStyle(
            // fontWeight: FontWeight.bold,
            fontSize: 14,
            color: Colors.black87),
      ),
      color: Colors.grey[400],
      /*  decoration: BoxDecoration(
                color: Colors.grey[400],
                shape: BoxShape.rectangle,
                borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(15.0),
                    bottomRight: Radius.circular(15.0))),*/
    );
  }

  Widget get addDetailsView {
    return Container(
      height: 60,
      margin: EdgeInsets.only(top: 90),
      padding: EdgeInsets.all(5),
      //width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Container(
            height: 50,
            width: 50,
            padding: EdgeInsets.all(5),
            child: ClipOval(
              child: Material(
                color: theme, // button color
                child: InkWell(
                  splashColor: Colors.white, // inkwell color
                  child: SizedBox(
                      width: 56,
                      height: 56,
                      child: Icon(Icons.add, color: Colors.white)),
                  onTap: () {
                    callPostAPI();
                  },
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  Future<void> callPostAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.ACCOUNT_URL;
    var objToSend = {
      "\"condition\"": [],
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    AccountDetailsBean accountDetailsResponseBean =
        AccountDetailsBean.fromJson(map);

    if (accountDetailsResponseBean.status) {
      accountDetailsBean = accountDetailsResponseBean;
      accountListBeanData = accountDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
      Toast.show(accountDetailsBean.message.toString(), context);
    }
  }

  Widget get detailsView {
    return Container(
      margin: EdgeInsets.only(top: 90),
      child: SlideListView(
        itemBuilder: (bc, index) {
          return InkWell(
            onTap: () {
              print("On Pressed" + index.toString());
              print("Name" + accountListBeanData[index].ownerId.email);

              Toast.show(accountListBeanData[index].email, context);

              Navigator.of(context).push(MaterialPageRoute(
                builder: (context) {
                  return ViewMoreScreen(
                    pageTitle: "Account Recent",
                    accountListBeanData: accountListBeanData[index],
                    theme: theme,
                  );
                },
              ));
            },
            child: BUICard.dashBoardCard(
              padding: 10,
              width: MediaQuery.of(context).size.width,
              cardSize: CardSize.large,
              cardTitle: accountListBeanData[index].email ?? "",
              cardName: accountListBeanData[index].companyname ?? "",
              statusTag: accountListBeanData[index].statusid.name ?? "",
              dashBoardCardTime:
                  accountListBeanData[index].createdAt.substring(0, 10) ?? "",
              height: 150,
              elevation: 10,
              isTagValue: false,
              //tagValue: [],
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(10.0),
              ),
              //tagValue: ["#one"],
            ),
          );

          /*      return Container(
              width: MediaQuery.of(context).size.width,
              padding: EdgeInsets.all(5),
              child: Card(
                elevation: 5,
                shape: BeveledRectangleBorder(
                  borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(15.0),
                    bottomRight: Radius.circular(15.0),
                    topLeft: Radius.circular(15.0),
                    topRight: Radius.circular(15.0),
                  ),
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Container(
                        width: MediaQuery.of(context).size.width,
                        decoration: BoxDecoration(
                            color: Colors.white,
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(15.0),
                                topRight: Radius.circular(15.0))),
                        padding: EdgeInsets.only(left: 20, right: 10, top: 15),
                        child: Text(
                          '${accountListBeanData[index].companyname.toString()}',
                          overflow: TextOverflow.ellipsis,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_20,
                              letterSpacing: 0.5,
                              color: AppColors.APP_COLOR_BLACK),
                        )),
                    Container(
                      padding: EdgeInsets.only(left: 20, right: 10, top: 10),
                      child: Text(
                        '${accountListBeanData[index].ownerId.name}',
                        style: TextStyle(
                            fontWeight: FontWeight.normal,
                            fontFamily: FontFamily.Gilroy,
                            fontSize: FontSize.FONT_15,
                            letterSpacing: 1,
                            color: AppColors.APP_COLOR_BLACK),
                      ),
                    ),
                    Container(
                        padding: EdgeInsets.only(left: 20, right: 10, top: 10),
                        child: Text(
                          '${accountListBeanData[index].email.toString()}',
                          style: TextStyle(
                              fontWeight: FontWeight.normal,
                              fontFamily: FontFamily.Gilroy,
                              fontSize: FontSize.FONT_15,
                              letterSpacing: 1,
                              color: AppColors.APP_COLOR_BLACK),
                        )),
                    Container(
                      padding: EdgeInsets.only(right: 15, top: 10, bottom: 10),
                      decoration: BoxDecoration(
                          color: Colors.white,
                          shape: BoxShape.rectangle,
                          borderRadius: BorderRadius.only(
                              bottomLeft: Radius.circular(15.0),
                              bottomRight: Radius.circular(15.0))),
                      alignment: Alignment.bottomRight,
                      child: Button.mini(
                        elevation: 1,
                        width: MediaQuery.of(context).size.width * 0.2,
                        onPressed: () {
                         */
          /* Navigator.of(context).push(MaterialPageRoute(
                            builder: (context) {
                              return ViewMoreScreen(
                                pageTitle: "Account Recent",
                                accountListBeanData: accountListBeanData[index],
                                theme: theme,
                              );
                            },
                          ));*/
          /*
                        },
                        btnTextColor: AppColors.appWhite,
                        btnColor: secondaryColorTheme ?? AppColors.APP_COLOR_PRIMARY,
                        btnText: AppStrings.VIEW_MORE,
                        buttonType: ButtonType.custom,
                      ),
                    ),
                  ],
                ),
              ));*/
        },
        actionWidgetDelegate:
            ActionWidgetDelegate(actions.length, (actionIndex, listIndex) {
          if (actionIndex == 0) {
            return Column(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                Icon(
                  Icons.edit,
                  color: theme ?? AppColors.primaryColor,
                ),
                Text(
                  actions[0].toString(),
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                    fontWeight: FontWeight.bold,
                    fontSize: 14,
                    color: theme ?? AppColors.primaryColor,
                  ),
                ),
              ],
            );
          } else {
            return Column(
              mainAxisSize: MainAxisSize.min,
              children: <Widget>[
                Icon(Icons.delete, color: AppColors.errorWarning),
                Text(
                  actions[1].toString(),
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 12,
                      color: AppColors.errorWarning),
                ),
              ],
            );
          }
        }, (int indexInList, int index, BaseSlideItem item) {
          if (index == 1) {
            CustomAlertBox.showCustomAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text("Are you sure do you want to delete this Item"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                item.remove();
                // Navigator.pop(context);
              },
            );
            //  _showMyDialog(indexInList, item);
          } else {
            item.close();
            CustomAlertBox.showCustomAlertBox(
              context: context,
              type: "Alert",
              enterWidget: Text("This is Sample Alert"),
              isPositiveEnabled: true,
              isNegativeEnabled: false,
              positiveButtonColor: Colors.green,
              setPositiveAction: () {},
            );
            Toast.show("Close this Item", context);
          }
        }, [Colors.white, Colors.white]),
        dataList: accountListBeanData,
        itemBackgroundColor: Colors.grey[200],
        refreshCallback: () async {
          await Future.delayed(Duration(seconds: 2));

          setState(() {
            // fetchData();

            callPostAPI();

            this.mounted;
          });
          return;
        },
      ),
    );
  }

  Future<void> onMenuPressed() async {
    setState(() {
      widget.onMenuPressed();

      widget.onMenuIconPressed();
      //   onMenuIconPressed();
    });
  }

  Future<void> onPopUpMenuPressed(int s) async {
    switch (s) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  void onBackPressed() {
    getStringValuesSF();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {
      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => CardStyle()));
    });
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Toast.show("Setting Pressed", context);

      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => ButtonView()));
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Toast.show("Notification Pressed", context);

      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => ChartsDemo()));
    });
  }
}
