import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/SignUpScreenComponent.dart';
import 'package:country_code_picker/country_code_picker.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Firebase Classes/sign_in.dart';
import 'package:example/Screen/MultiStageScreen.dart';
import 'package:example/main.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';

import 'LoginScreen.dart';
import 'MenuScreen.dart';
import 'OtpScreen.dart';

class SignUpScreen extends StatefulWidget {
  static String tag = AppStrings.SIGN_UP_TAG;

  @override
  SignUpScreenState createState() => SignUpScreenState();
}

class SignUpScreenState extends State<SignUpScreen> {


  //controllers
  final phoneNumberController = TextEditingController();
  final emailController = TextEditingController();
  final passwordController = TextEditingController();
  final firstNameController = TextEditingController();
  final lastNameController = TextEditingController();

  //tab bar options
  List<String> tabBarName = [
    AppStrings.SOCIAL,
    AppStrings.MOBILE,
    AppStrings.EMAIL
  ];

  //child variables
  String phoneNumber;
  String countryCode = "+91";
  String phoneNo, verificationId, smsCode;
  bool codeSent = false;
  static String user_name;
  static String user_email;
  static bool user_emailVerified;
  static String user_imageUrl;
  static String user_token;
  static String user_uid;
  static SharedPreferences sharedPreferences;

  //Firebase Auth instance

  final FirebaseAuth __firebaseAuth = FirebaseAuth.instance;

  //formkey
  var _formKey = GlobalKey<FormState>();
  var _phoneGlobalKey = GlobalKey<FormState>();

  //facebook login
  // static final FacebookLogin facebookSignIn = new FacebookLogin();

  CountryCode code;
  SharedPreferences prefs;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    //_onCountryChange(code);
    getStringValuesSF();
  }

  @override
  void dispose() {
    // Clean up the controller when the widget is removed from the
    // widget tree.
    phoneNumberController.dispose();
    emailController.dispose();
    passwordController.dispose();
    firstNameController.dispose();
    lastNameController.dispose();

    super.dispose();
  }

  Color theme = AppColors.APP_COLOR_PRIMARY;

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    //Return int
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: SignUpScreenComponent(
        googleSignUp: googleSignUp,
        facebookSignUp: facebookSignUp,
        phoneNumberController: phoneNumberController,
        phoneNumberSignInFunction: phoneNumberSignInFunction,
        globalKey: _formKey,
        phoneGlobalKey: _phoneGlobalKey,
        emailController: emailController,
        passwordController: passwordController,
        firstNameController: firstNameController,
        lastNameController: lastNameController,
        emailPasswordSignInFunction: emailPasswordSignInFunction,
        signInFunction: signInPage,
        tabEnabledName: tabBarName,
        themeColor: theme,
        imageLogo: Container(
          height: MediaQuery.of(context).size.height * 0.3,
          width: MediaQuery.of(context).size.width * 0.5,
          child: CircleAvatar(
            radius: 20,
            backgroundColor: Colors.white,
            child: Container(
              padding: EdgeInsets.all(20),
              child: Image.asset(
                ImageAssets.LOGO_WITH_NAME,
                height: MediaQuery.of(context).size.height * 0.2,
              ),
            ),
          ),
        ),
      ),
    );
  }

  Future<void> googleSignUp() async {
    signInWithGoogle().then((result) {
      if (result != null) {
        user_email = email;
        user_name = name;
        user_emailVerified = emailVerified;
        user_imageUrl = imageUrl;
        user_token = "Bearer " + bearToken.toString();
        user_uid = uid;

        MyHomePageState.user_token = user_token;

        prefs.setString(AppStrings.EMAIL, email);

        Navigator.of(context).push(
          MaterialPageRoute(
            builder: (context) {
              return MenuScreen();
            },
          ),
        );

        Toast.show("Login Success" + name, context);
      } else {
        Toast.show("Login Failed", context);
      }
    }).catchError((onError) {});
  }

  Future<void> facebookSignUp() async {
    Toast.show("Facebook Sign in Method Calling", context);
    Navigator.of(context).push(MaterialPageRoute(
      builder: (context) {
        return MultiStageScreen();
      },
    ));
    //facebookLogin();
  }

  Future<void> emailPasswordSignInFunction() async {
    final isValid = _formKey.currentState.validate();
    if (!isValid) {
      return;
    }
    _formKey.currentState.save();
    signUp(emailController.text.toString(), passwordController.text.toString());
  }

  Future<void> signUp(String email, String password) async {
    __firebaseAuth
        .createUserWithEmailAndPassword(email: email, password: password)
        .then((result) {
      if (result != null) {
        Navigator.of(context).push(
          MaterialPageRoute(
            builder: (context) {
              return MenuScreen();
            },
          ),
        );
        Toast.show("Login Success", context);
      } else {
        Toast.show("Login Failed", context);
      }
    });
  }

  Future<void> phoneNumberSignInFunction() async {
    final isValid = _phoneGlobalKey.currentState.validate();
    if (!isValid) {
      return;
    }
    _phoneGlobalKey.currentState.save();
    phoneNumber = phoneNumberController.text.toString();

    if (phoneNumber.isNotEmpty) {
      if (phoneNumber.length == 10) {
        //verifyPhone(phoneNumber);
        Navigator.pushReplacement(context,
            MaterialPageRoute(builder: (BuildContext context) => OtpScreen()));
      } else {}
    } else {}
  }

  Future<void> signInPage() async {
    Navigator.pushReplacement(context,
        MaterialPageRoute(builder: (BuildContext context) => LoginScreen()));
  }

  Future<void> verifyPhone(String num) async {
    final PhoneCodeAutoRetrievalTimeout autoRetrievalTimeout = (String verId) {
      this.verificationId = verId;
    };

    final PhoneCodeSent smsCodeSent = (String verId, [int forceCodeResend]) {
      this.verificationId = verId;
    };

    final PhoneVerificationCompleted verifiedSuccess =
        (PhoneAuthCredential user) {};

    final PhoneVerificationFailed verifiedFailed =
        (FirebaseAuthException user) {};

    await FirebaseAuth.instance.verifyPhoneNumber(
      phoneNumber: countryCode + " " + num,
      codeAutoRetrievalTimeout: autoRetrievalTimeout,
      codeSent: smsCodeSent,
      timeout: const Duration(seconds: 10),
      verificationCompleted: verifiedSuccess,
      verificationFailed: verifiedFailed,
    );
  }

/*  Future<void> facebookLogin() async {
    final FacebookLoginResult result = await facebookSignIn.logIn(['email']);

    switch (result.status) {
      case FacebookLoginStatus.loggedIn:
        final FacebookAccessToken accessToken = result.accessToken;

        break;
      case FacebookLoginStatus.cancelledByUser:

        break;
      case FacebookLoginStatus.error:

        break;
    }
  }

  Future<void> _logOut() async {
    await facebookSignIn.logOut();

  }*/


}
