import 'dart:io';

import 'package:flutter/material.dart';

enum SNACK_BAR_TYPE { SUCCESS, FAILURE, WARNING }

class Utils {
  static void hideKeyboard(BuildContext context) =>
      FocusScope.of(context).requestFocus(new FocusNode());

  static Widget setImageView(String assetUrl,
          {double valWidth = 90.0, double valHeight = 38.0}) =>
      new Container(
          child: new Image.asset(assetUrl, width: valWidth, height: valHeight));

  static void showInSnackBar(
      {GlobalKey<ScaffoldState> scaffoldKey,
      String message,
      SNACK_BAR_TYPE snackBarType}) {
    Color bgColor = Colors.grey;
    if (SNACK_BAR_TYPE.SUCCESS == snackBarType)
      bgColor = Colors.green;
    else if (SNACK_BAR_TYPE.FAILURE == snackBarType)
      bgColor = Colors.red;
    else if (SNACK_BAR_TYPE.WARNING == snackBarType)
      bgColor = Colors.orange;
    else
      bgColor = Colors.grey;
    scaffoldKey.currentState.showSnackBar(new SnackBar(
      duration: Duration(seconds: 3),
      content: new Text(
        message,
      ),
      backgroundColor: bgColor,
    ));
  }

  static void showSnackBar(BuildContext context, String value) {
    Scaffold.of(context).showSnackBar(
      SnackBar(
        content: Text(value),
      ),
    );
  }

  static Future<bool> isNetworkConnected() async {
    try {
      final result = await InternetAddress.lookup('google.com');
      if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
        return true;
      }
    } on SocketException catch (_) {
      return false;
    }
  }

  static bool equalsIgnoreCase(String str1, String str2) =>
      (str1 == null && str2 == null) ||
      (str1 != null &&
          str2 != null &&
          str1.toLowerCase() == str2.toLowerCase());
}
