import 'package:bui_component/Constants/ImageAssets.dart';
import 'package:flutter/material.dart';
import 'buttonType.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/SvgPictures.dart';
import 'package:flutter_svg/svg.dart';

// ignore: must_be_immutable
class Button extends StatelessWidget {
  //required
  /// [buttonType] sets the style and icons of the button.
  ButtonType buttonType;

  //required
  /// [onPressed] Send a function to trigger the button.
  Function onPressed;

  //not required, default left
  /// [imagePosition] set the position of the icon.(left or right)
  ImagePosition imagePosition;

  //not required, default 5.0
  /// [elevation] set the button's elevation value.
  double elevation;

  //not required, default small
  /// [buttonSize] set the size of the button. (small medium large)
  ButtonSize buttonSize;

  //not required, Gets value according to buttonType.
  /// [btnColor] Set the background color of the button.
  Color btnColor;

  //not required, Gets value according to buttonType.
  /// [btnTextColor] set the button's text color.
  Color btnTextColor;

  //not required, Gets value according to buttonType.
  /// [btnText] set the button's text.
  String btnText;

  //not required, Gets value according to buttonSize.
  /// You can change the value of [width] when the text size becomes too small.
  double width;

  //not required, Gets value according to buttonSize.
  /// [padding] set the button's padding value.
  double padding;

  /// [_image] value cannot be assigned.Gets value according to [buttonType].
  Widget _image;

  /// [_fontSize] value cannot be assigned.Gets value according to [buttonSize].
  double _fontSize;

  /// [_imageSize] value cannot be assigned.Gets value according to [buttonSize].
  double _imageSize;

  //not required, button shape.
  /// [shape] set the button's shape.
  ShapeBorder shape;

  //not required, button model.
  /// [mini] It automatically takes value according to the selected constructor.
  bool mini;

  Button({
    this.buttonType,
    this.onPressed,
    this.imagePosition: ImagePosition.left,
    this.buttonSize: ButtonSize.small,
    this.btnColor,
    this.btnTextColor,
    this.btnText,
    this.elevation: 5.0,
    this.width,
    this.padding,
    this.shape,
  }) : mini = false;

  Button.mini({
    this.buttonType,
    this.onPressed,
    this.buttonSize: ButtonSize.small,
    this.btnColor,
    this.elevation: 5.0,
    this.padding,
    this.imagePosition,
    this.btnText,
    this.btnTextColor,
    this.shape,
    this.width,
  }) : mini = true;

  @override
  Widget build(BuildContext context) {
    _setButtonSize();
    _createStyle();
    return mini
        ? MaterialButton(
            onPressed: onPressed as void Function(),
            elevation: elevation,
            shape: shape ??
                RoundedRectangleBorder(
                    borderRadius: BorderRadius.all(Radius.circular(5.0))),
            color: btnColor,
            child: Container(
              width: width,
              child: _image != null
                  ? Row(
                      mainAxisSize: MainAxisSize.min,
                      mainAxisAlignment: imagePosition == ImagePosition.left
                          ? MainAxisAlignment.center
                          : MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        _image != null ? Container(child: _image) : Container(),
                        Text(
                          btnText,
                          style: TextStyle(color: btnTextColor),
                        ),
                      ],
                    )
                  : Text(
                      btnText,
                      style: TextStyle(color: btnTextColor),
                      textAlign: TextAlign.center,
                    ),
            ))
        : MaterialButton(
            onPressed: onPressed as void Function(),
            color: btnColor,
            child: _image,
            elevation: elevation,
            padding: EdgeInsets.all(padding),
            shape: CircleBorder(),
          );
  }

  void _setButtonSize() {
    if (buttonSize == ButtonSize.small) {
      padding ??= mini ? 5.0 : 6.0;
      width ??= 160;
      _fontSize = 15.0;
      _imageSize = mini ? 34.0 : 40.0;
    } else if (buttonSize == ButtonSize.medium) {
      padding ??= mini ? 5.5 : 6.5;
      width ??= 220;
      _fontSize = 17.0;
      _imageSize = mini ? 40.0 : 46.0;
    } else {
      padding ??= mini ? 6.0 : 7.0;
      width ??= 250;
      _fontSize = 19.0;
      _imageSize = mini ? 46.0 : 52.0;
    }
  }

  void _createStyle() {
    switch (buttonType) {
      case ButtonType.facebook:
        btnText ??= 'Sign in with Facebook';
        btnTextColor ??= Colors.white;
        btnColor ??= AppColors.FACEBOOK_COLOR;
        _image = SvgPicture.asset(
          SvgPictures.FB_SVG,
          height: _imageSize,
          width: _imageSize,
        );
        /*Image.asset(
          ImageAssets.FACEBOOK,
          width: _imageSize,
          height: 150,
        );*/
        break;

      case ButtonType.facebookDark:
        btnText ??= 'Sign in with Facebook';
        btnTextColor ??= Colors.white;
        btnColor ??= Colors.black;
        _image = Image.asset(
          ImageAssets.FACEBOOK_DARK,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.github:
        btnText ??= 'Sign in with Github';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        _image = Image.asset(
          ImageAssets.GITHUB,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.pinterest:
        btnText ??= 'Sign in with Pinterest';
        btnTextColor ??= Colors.white;
        btnColor ??= Colors.redAccent;
        _image = Image.asset(
          ImageAssets.PINTEREST,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.apple:
        btnText ??= 'Sign in with Apple';
        btnTextColor ??= Colors.black;
        btnColor ??= Color(0xfff7f7f7);
        _image = Image.asset(
          ImageAssets.APPLE,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.twitter:
        btnText ??= 'Sign in with Twitter';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF1DA1F2);
        _image = Image.asset(
          ImageAssets.TWITTER,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.linkedin:
        btnText ??= 'Sign in with LinkedIn';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF3282B8);
        _image = Image.asset(
          ImageAssets.LINKED_IN,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.google:
        btnText ??= 'Sign in with Google';
        btnTextColor ??= Colors.black;
        btnColor ??= Color(0xfff7f7f7);
        _image = Image.asset(
          ImageAssets.GMAIL,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.googleDark:
        btnText ??= 'Sign in with Google';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF4285F4);
        _image = Image.asset(
          ImageAssets.GMAIL_DARK,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.youtube:
        btnText ??= 'Sign in with Youtube';
        btnTextColor ??= Colors.black;
        btnColor ??= Colors.white;
        _image = Image.asset(
          ImageAssets.YOUTUBE,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.microsoft:
        btnText ??= 'Sign in with Microsoft';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF2F2F2F);
        _image = Image.asset(
          ImageAssets.MICROSOFT,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.tumblr:
        btnText ??= 'Sign in with Tumblr';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF0F4C75);
        _image = Image.asset(
          ImageAssets.TUMBIR,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.mail:
        btnText ??= 'Sign in with Mail';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF20639B);
        _image = Image.asset(
          ImageAssets.MAIL,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.reddit:
        btnText ??= 'Sign in with Reddit';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFFC85417);
        _image = Image.asset(
          ImageAssets.REDDIT,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.yahoo:
        btnText ??= 'Sign in with Yahoo';
        btnTextColor ??= Colors.white;
        btnColor ??= Color(0xFF7C5295);
        _image = Image.asset(
          ImageAssets.YAHOO,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.amazon:
        btnText ??= 'Sign in with Amazon';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        _image = Image.asset(
          ImageAssets.AMAZON,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.quora:
        btnText ??= 'Sign in with Quora';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        _image = Image.asset(
          ImageAssets.QUORA,
          width: _imageSize,
          height: _imageSize,
        );
        break;

      case ButtonType.okay:
        btnText ??= 'OKAY';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;

        break;
      case ButtonType.cancel:
        btnText ??= 'CANCEL';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;

        break;
      case ButtonType.accept:
        btnText ??= 'ACCEPT';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;

        break;
      case ButtonType.delete:
        btnText ??= 'DELETE';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        break;
      case ButtonType.save:
        btnText ??= 'SAVE';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;

        break;
      case ButtonType.exit:
        btnText ??= 'EXIT';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;

        break;

      case ButtonType.done:
        btnText ??= 'DONE';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        break;

      case ButtonType.custom:
        btnText ??= 'Okay';
        btnTextColor ??= Colors.black87;
        btnColor ??= Colors.white;
        break;
    }
  }
}
