import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';

class BUICard extends StatelessWidget {
  CardType cardType;

  double elevation;

  CardSize cardSize;

  Color cardColor;

  double width;

  double height;

  double padding;

  Widget image;

  double fontSize;

  double imageSize;

  //not required, button shape.
  ShapeBorder shape;

  String cardText;
  String cardTitle;
  String cardName;
  String statusTag;
  String dashBoardCardTime;

  String cardPrice;
  String cardPercentage;
  String cardDate;
  bool trandValue;

  bool isTagValue;
  List<String> tagValue = [];

  //TaskCard String Values

  String cardStatus;
  String cardDesc;
  String cardAuthor;

  Color secondaryColor;

  //not required, Gets value according to buttonType.

  Color cardTextColor;

  bool mini;

  bool titleCard;

  bool dashBoardCard;

  bool dashBoardCardDeals;

  bool taskCard;

  bool notesCard;

  Function() onStateChange;

  bool isActiveStatus = false;

  String firstHalf;
  String secondHalf;

  bool flag;

  BUICard({
    this.cardType,
    this.elevation,
    this.cardSize,
    this.cardColor,
    this.width,
    this.height,
    this.padding,
    this.shape,
    this.cardText,
    this.cardTextColor,
  })  : mini = false,
        dashBoardCardDeals = false,
        titleCard = false,
        dashBoardCard = false;

  BUICard.mini({
    this.cardType,
    this.elevation,
    this.cardSize,
    this.cardColor,
  })  : mini = true,
        titleCard = false;

  BUICard.titleCard({
    this.cardTitle,
    this.cardName,
    this.cardType,
    this.elevation,
    this.cardSize,
    this.cardColor,
    this.width,
    this.height,
    this.padding,
    this.shape,
    this.cardText,
    this.cardTextColor,
  })  : titleCard = true,
        mini = false;

  BUICard.dashBoardCard({
    this.cardTitle,
    this.statusTag,
    this.cardName,
    this.cardType,
    this.dashBoardCardTime,
    this.elevation,
    this.cardSize,
    this.cardColor,
    this.width,
    this.height,
    this.padding,
    this.shape,
    this.cardTextColor,
    this.tagValue,
    this.isTagValue,
  })  : titleCard = false,
        dashBoardCard = true,
        mini = false;

  BUICard.dashBoardCardDeals({
    this.padding,
    this.cardSize,
    this.height,
    this.width,
    this.cardTitle,
    this.cardType,
    this.cardPrice,
    this.cardPercentage,
    this.cardDate,
    this.elevation,
    this.trandValue,
    this.cardTextColor,
    this.cardColor,
    this.shape,
  })  : dashBoardCardDeals = true,
        titleCard = false,
        dashBoardCard = false,
        mini = false;

  BUICard.taskCard({
    this.cardStatus,
    this.cardDesc,
    this.cardDate,
    this.cardType,
    this.cardSize,
    this.cardColor,
    this.cardTextColor,
    this.shape,
    this.elevation,
    this.width,
    this.height,
    this.padding,
    this.onStateChange,
    this.isActiveStatus,
    this.secondaryColor,
  })  : taskCard = true,
        dashBoardCardDeals = false,
        titleCard = false,
        dashBoardCard = false,
        mini = false;

  BUICard.notesCard({
    this.cardAuthor,
    this.cardDesc,
    this.cardDate,
    this.cardType,
    this.cardSize,
    this.cardColor,
    this.cardTextColor,
    this.shape,
    this.elevation,
    this.width,
    this.height,
    this.padding,
    this.secondaryColor,
    this.onStateChange,
    this.flag,
  })  : notesCard = true,
        taskCard = false,
        dashBoardCardDeals = false,
        titleCard = false,
        dashBoardCard = false,
        mini = false;

  @override
  Widget build(BuildContext context) {
    _setCardSize();
    _createStyle();

    if (mini) {
      return Container(
        child: Card(
          shape: shape,
          color: cardColor,
          child: Text(
            cardText,
            style: TextStyle(letterSpacing: 1, color: cardTextColor),
          ),
          elevation: elevation,
        ),
      );
    } else if (titleCard) {
      return Container(
        width: width,
        height: height,
        child: Card(
          shape: shape,
          color: cardColor,
          elevation: elevation,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                padding: EdgeInsets.all(padding),
                child: Text(
                  cardTitle,
                  style: TextStyle(
                      letterSpacing: 1,
                      color: cardTextColor,
                      fontSize: FontSize.FONT_19,
                      fontWeight: FontWeight.bold,
                      fontFamily: FontFamily.Gilroy),
                ),
              ),
              Container(
                padding: EdgeInsets.all(padding),
                child: Text(
                  cardName,
                  style: TextStyle(
                      letterSpacing: 1,
                      color: cardTextColor,
                      fontSize: FontSize.FONT_15,
                      fontWeight: FontWeight.w300,
                      fontFamily: FontFamily.Gilroy),
                ),
              ),
              Container(
                padding: EdgeInsets.all(padding),
                child: Text(
                  cardText,
                  style: TextStyle(
                      letterSpacing: 1,
                      color: cardTextColor,
                      fontSize: FontSize.FONT_13,
                      fontWeight: FontWeight.w300,
                      fontFamily: FontFamily.Gilroy),
                ),
              ),
            ],
          ),
        ),
      );
    } else if (dashBoardCard) {
      return Container(
        padding: EdgeInsets.all(padding),
        width: width,
        height: height,
        child: Card(
          shape: shape,
          color: cardColor,
          elevation: elevation,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Container(
                    width: 200,
                    padding: EdgeInsets.only(left: 10, top: 10),
                    child: Text(
                      cardTitle,
                      softWrap: false,
                      style: TextStyle(
                          letterSpacing: 1,
                          color: cardTextColor,
                          fontSize: FontSize.FONT_19,
                          fontWeight: FontWeight.bold,
                          fontFamily: FontFamily.Gilroy),
                    ),
                  ),
                  Container(
                    padding: EdgeInsets.only(left: 5, right: 5),
                    alignment: Alignment.center,
                    height: 20,
                    width: 70,
                    margin: EdgeInsets.only(right: 10, top: 10),
                    decoration: BoxDecoration(
                      color: AppColors.LIGHT_GREY,
                      borderRadius: BorderRadius.circular(5),
                    ),
                    child: Text(
                      statusTag,
                      textAlign: TextAlign.center,
                      softWrap: false,
                      style: TextStyle(
                          letterSpacing: 1,
                          color: AppColors.APP_COLOR_BLACK,
                          fontSize: FontSize.FONT_12,
                          fontWeight: FontWeight.w700,
                          fontFamily: FontFamily.Gilroy),
                    ),
                  ),
                ],
              ),
              Container(
                padding: EdgeInsets.only(left: 5, top: 5),
                margin: EdgeInsets.only(left: 5, top: 5),
                child: Text(
                  cardName,
                  style: TextStyle(
                      letterSpacing: 1,
                      color: cardTextColor,
                      fontSize: FontSize.FONT_15,
                      fontWeight: FontWeight.w300,
                      fontFamily: FontFamily.Gilroy),
                ),
              ),
              if (isTagValue)
                if (tagValue.isNotEmpty)
                  Container(
                    height: 30,
                    padding: EdgeInsets.only(left: 1, right: 5),
                    margin: EdgeInsets.only(left: 10, top: 1),
                    child: ListView.builder(
                        shrinkWrap: true,
                        scrollDirection: Axis.horizontal,
                        itemCount: tagValue.length,
                        itemBuilder: (BuildContext context, int index) {
                          return Container(
                            /*padding: EdgeInsets.only(left: 3, right: 5),*/
                            margin: EdgeInsets.only(right: 10, top: 10),
                            child: Container(
                              alignment: Alignment.center,
                              height: 15,
                              width: 100,
                              decoration: BoxDecoration(
                                color: AppColors.LIGHT_GREY,
                                borderRadius: BorderRadius.circular(4),
                              ),
                              child: Text(
                                //"#"+index.toString(),
                                tagValue[index],
                                softWrap: true,
                                maxLines: 1,
                                style: TextStyle(
                                    letterSpacing: 1,
                                    color: AppColors.APP_COLOR_BLACK,
                                    fontSize: FontSize.FONT_13,
                                    fontWeight: FontWeight.w700,
                                    fontFamily: FontFamily.Gilroy),
                              ),
                            ),
                          );
                        }),
                  ),
              if (dashBoardCardTime.isNotEmpty)
                Container(
                  // padding: EdgeInsets.only(left: 5, right: 10),
                  margin: EdgeInsets.only(left: 10, top: 10),
                  alignment: Alignment.centerRight,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.end,
                    children: [
                      Container(
                        //padding: EdgeInsets.all(10),
//                      margin: EdgeInsets.only(left: 15),
                        child: Icon(
                          Icons.calendar_today_rounded,
                          size: 20,
                          color: AppColors.APP_COLOR_PRIMARY,
                        ),
                      ),
                      Container(
                        margin: EdgeInsets.only(left: 10, top: 1, right: 5),
                        padding: EdgeInsets.only(top: 1, right: 10),
                        //padding: EdgeInsets.all(5),
                        child: Text(
                          dashBoardCardTime,
                          softWrap: true,
                          maxLines: 1,
                          style: TextStyle(
                              letterSpacing: 1,
                              color: AppColors.APP_COLOR_BLACK,
                              fontSize: FontSize.FONT_13,
                              fontWeight: FontWeight.normal,
                              fontFamily: FontFamily.Gilroy),
                        ),
                      ),
                    ],
                  ),
                ),
            ],
          ),
        ),
      );
    } else if (dashBoardCardDeals) {
      return Container(
        margin: EdgeInsets.all(2),
        height: height,
        width: width,
        padding: EdgeInsets.all(padding),
        child: Card(
          elevation: elevation,
          shape: shape,
          color: cardColor,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Container(
                padding: EdgeInsets.only(left: 10, top: 10, bottom: 0),
                child: Text(
                  cardTitle,
                  style: TextStyle(
                    color: cardTextColor ?? AppColors.APP_COLOR_PRIMARY,
                    fontSize: FontSize.FONT_16,
                    fontFamily: FontFamily.Gilroy,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              Column(
                mainAxisAlignment: MainAxisAlignment.end,
                crossAxisAlignment: CrossAxisAlignment.end,
                children: [
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Container(
                        padding: EdgeInsets.only(left: 10, top: 15),
                        child: Text(
                          cardPrice,
                          style: TextStyle(
                            color: cardTextColor ?? AppColors.APP_COLOR_PRIMARY,
                            fontSize: FontSize.FONT_16,
                            fontFamily: FontFamily.Gilroy,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                      ),
                      Container(
                        padding: EdgeInsets.only(right: 30, top: 15),
                        child: Row(
                          children: [
                            Icon(
                              trandValue
                                  ? Icons.arrow_upward_sharp
                                  : Icons.arrow_downward_sharp,
                              color: trandValue
                                  ? Colors.greenAccent
                                  : Colors.redAccent,
                            ),
                            Container(
                              child: Text(
                                cardPercentage,
                                style: TextStyle(
                                  color: trandValue
                                      ? Colors.greenAccent
                                      : Colors.redAccent,
                                  fontSize: FontSize.FONT_16,
                                  fontFamily: FontFamily.Gilroy,
                                  fontWeight: FontWeight.bold,
                                ),
                              ),
                            ),
                          ],
                        ),
                      )
                    ],
                  ),
                  Container(
                    padding: EdgeInsets.only(right: 15),
                    child: Text(
                      cardDate,
                      style: TextStyle(
                        color:
                            trandValue ? Colors.greenAccent : Colors.redAccent,
                        fontSize: FontSize.FONT_10,
                        fontFamily: FontFamily.Gilroy,
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                  ),
                ],
              ),
            ],
          ),
        ),
      );
    } else if (taskCard) {
      return Container(
          margin: EdgeInsets.all(2),
          height: height,
          width: width,
          padding: EdgeInsets.all(padding),
          child: Card(
            shape: shape,
            elevation: elevation,
            color: cardColor,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  children: [
                    InkWell(
                      onTap: () {
                        onStateChange();
                      },
                      child: Container(
                          padding: EdgeInsets.only(
                            top: 10,
                            left: 15,
                            bottom: 5,
                            right: 10,
                          ),
                          child: Icon(
                            Icons.check_circle_rounded,
                            color: isActiveStatus
                                ? secondaryColor != null
                                    ? secondaryColor
                                    : Colors.greenAccent
                                : AppColors.APP_GRAY_1,
                          )),
                    ),
                    Container(
                      padding: EdgeInsets.only(
                        top: 10,
                        left: 15,
                        bottom: 5,
                        right: 10,
                      ),
                      child: Text(
                        cardStatus,
                        style: TextStyle(
                          color: cardTextColor != null
                              ? cardTextColor
                              : Colors.black87,
                          fontSize: FontSize.FONT_20,
                          fontFamily: FontFamily.Gilroy,
                          fontWeight: FontWeight.bold,
                          letterSpacing: 1.0,
                          wordSpacing: 1.0,
                        ),
                        softWrap: true,
                        maxLines: 2,
                      ),
                    ),
                  ],
                ),
                Container(
                  padding: EdgeInsets.only(
                    top: 5,
                    left: 66,
                    bottom: 5,
                    right: 5,
                  ),
                  //margin:EdgeInsets.all(2),
                  child: Text(
                    cardDesc,
                    style: TextStyle(
                      color: cardTextColor != null
                          ? cardTextColor
                          : Colors.black87,
                      fontSize: FontSize.FONT_15,
                      fontFamily: FontFamily.Gilroy,
                      fontWeight: FontWeight.normal,
                      letterSpacing: 1.0,
                      wordSpacing: 1.0,
                    ),
                    softWrap: true,
                    maxLines: 2,
                  ),
                ),
                Container(
                  padding: EdgeInsets.only(
                    top: 5,
                    left: 66,
                    bottom: 5,
                    right: 5,
                  ),
                  child: Text(
                    cardDate,
                    style: TextStyle(
                      color: cardTextColor != null
                          ? cardTextColor
                          : Colors.black87,
                      fontSize: FontSize.FONT_13,
                      fontFamily: FontFamily.Gilroy,
                      fontWeight: FontWeight.normal,
                      letterSpacing: 1.0,
                      wordSpacing: 1.0,
                    ),
                    softWrap: true,
                    maxLines: 2,
                  ),
                ),
              ],
            ),
          ));
    } else if (notesCard) {
      print("Card Length====" + cardDesc.length.toString());

      if (cardDesc.length > 50) {
        print("If====");
        firstHalf = cardDesc.substring(0, 50);
        secondHalf = cardDesc.substring(50, cardDesc.length);
      } else {
        print("Else====");
        firstHalf = cardDesc;
        secondHalf = "";
      }
      /*   return Container(
          margin: EdgeInsets.all(2),
          height: height,
          width: width,
          padding: EdgeInsets.all(padding),
          child: Card(
            shape: shape,
            elevation: elevation,
            color: cardColor,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Container(
                  padding: EdgeInsets.only(
                    top: 10,
                    left: 60,
                    right: 20,
                  ),
                  child: Text(
                    cardAuthor,
                    style: TextStyle(
                      color: cardTextColor != null
                          ? cardTextColor
                          : Colors.black87,
                      fontSize: FontSize.FONT_17,
                      fontFamily: FontFamily.Gilroy,
                      fontWeight: FontWeight.bold,
                      letterSpacing: 1.0,
                      wordSpacing: 1.0,
                    ),
                    softWrap: true,
                    maxLines: 2,
                  ),
                ),
                Row(
                  children: [
                    Container(
                        padding: EdgeInsets.only(
                          top: 5,
                          left: 15,
                          bottom: 5,
                          right: 10,
                        ),
                        child: Icon(Icons.sticky_note_2_rounded,
                            color: secondaryColor != null
                                ? secondaryColor
                                : Colors.greenAccent)),
                    */
      /*  Container(
                      padding: EdgeInsets.only(
                        top: 10,
                        left: 10,
                        bottom: 5,
                        right: 5,
                      ),
                      width: MediaQuery.of(context).size.width * 0.7,
                      //margin:EdgeInsets.all(2),
                      child: Text(
                        cardDesc,
                        style: TextStyle(
                          color: cardTextColor != null
                              ? cardTextColor
                              : Colors.black87,
                          fontSize: FontSize.FONT_15,
                          fontFamily: FontFamily.Gilroy,
                          fontWeight: FontWeight.bold,
                          letterSpacing: 1.0,
                          wordSpacing: 1.0,
                        ),
                        softWrap: true,
                        maxLines: 2,
                      ),
                    ),*/
      /*

                    secondHalf.isEmpty
                        ? new Text(firstHalf)
                        : new Column(
                            children: <Widget>[
                              new Text(flag
                                  ? (firstHalf + "...")
                                  : (firstHalf + secondHalf)),
                              new InkWell(
                                child: new Row(
                                  mainAxisAlignment: MainAxisAlignment.end,
                                  children: <Widget>[
                                    new Text(
                                      flag ? "show more" : "show less",
                                      style: new TextStyle(color: Colors.blue),
                                    ),
                                  ],
                                ),
                                onTap: () {
                                  onStateChange();
                                },
                              ),
                            ],
                          ),
                  ],
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Container(
                      padding: EdgeInsets.only(
                        top: 5,
                        left: 60,
                        bottom: 5,
                        right: 5,
                      ),
                      child: Text(
                        cardDate,
                        style: TextStyle(
                          color: cardTextColor != null
                              ? cardTextColor
                              : Colors.black87,
                          fontSize: FontSize.FONT_13,
                          fontFamily: FontFamily.Gilroy,
                          fontWeight: FontWeight.normal,
                          letterSpacing: 1.0,
                          wordSpacing: 1.0,
                        ),
                        softWrap: true,
                        maxLines: 2,
                      ),
                    ),
                    Container(
                      padding: EdgeInsets.only(
                        top: 5,
                        left: 5,
                        bottom: 5,
                        right: 20,
                      ),
                      child: Text(
                        AppStrings.VIEW_MORE,
                        style: TextStyle(
                          color: cardTextColor != null
                              ? cardTextColor
                              : Colors.black87,
                          fontSize: FontSize.FONT_13,
                          fontFamily: FontFamily.Gilroy,
                          fontWeight: FontWeight.normal,
                          letterSpacing: 1.0,
                          wordSpacing: 1.0,
                        ),
                        softWrap: true,
                        maxLines: 2,
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ));*/

      return Card(
        child: Container(
          padding: new EdgeInsets.symmetric(horizontal: 5.0, vertical: 5.0),
          child: secondHalf.isEmpty
              ? Container(
                  height: height,
                  child: Column(
                    children: [
                      Row(
                        children: [
                          Container(
                              padding: EdgeInsets.only(
                                top: 10,
                                left: 15,
                                bottom: 5,
                                right: 10,
                              ),
                              child: Icon(Icons.sticky_note_2_rounded,
                                  size: 20,
                                  color: secondaryColor != null
                                      ? secondaryColor
                                      : Colors.greenAccent)),
                          Container(
                            alignment: Alignment.topLeft,
                            padding: EdgeInsets.only(
                              top: 10,
                              left: 10,
                              right: 20,
                              //bottom: 10,
                            ),
                            child: Text(
                              cardAuthor,
                              style: TextStyle(
                                color: cardTextColor != null
                                    ? cardTextColor
                                    : Colors.black87,
                                fontSize: FontSize.FONT_17,
                                fontFamily: FontFamily.Gilroy,
                                fontWeight: FontWeight.bold,
                                letterSpacing: 1.0,
                                wordSpacing: 1.0,
                              ),
                              textAlign: TextAlign.center,
                              softWrap: true,
                              maxLines: 2,
                            ),
                          ),
                        ],
                      ),
                      Container(
                          padding: EdgeInsets.all(10),
                          alignment: Alignment.topLeft,
                          child: new Text(
                            firstHalf,
                            textAlign: TextAlign.start,
                            style: TextStyle(
                              color: cardTextColor != null
                                  ? cardTextColor
                                  : Colors.black87,
                              fontSize: FontSize.FONT_15,
                              fontFamily: FontFamily.Gilroy,
                              fontWeight: FontWeight.w600,
                              letterSpacing: 1.0,
                              wordSpacing: 1.0,
                            ),

                            //softWrap: true,
                            //maxLines: 2,
                          )),
                      Container(
                        alignment: Alignment.topLeft,
                        padding: EdgeInsets.only(
                          //   top: 10,
                          left: 10,
                          right: 40,
                          bottom: 10,
                        ),
                        child: Text(
                          cardDate,
                          style: TextStyle(
                            color: cardTextColor != null
                                ? cardTextColor
                                : Colors.black87,
                            fontSize: FontSize.FONT_13,
                            fontFamily: FontFamily.Gilroy,
                            fontWeight: FontWeight.normal,
                            letterSpacing: 1.0,
                            wordSpacing: 1.0,
                          ),
                          textAlign: TextAlign.center,
                          softWrap: true,
                          maxLines: 2,
                        ),
                      ),
                    ],
                  ),
                )
              : new Column(
                  children: <Widget>[
                    Row(
                      children: [
                        Container(
                            padding: EdgeInsets.only(
                              top: 10,
                              left: 15,
                              bottom: 5,
                              right: 10,
                            ),
                            child: Icon(Icons.sticky_note_2_rounded,
                                size: 20,
                                color: secondaryColor != null
                                    ? secondaryColor
                                    : Colors.greenAccent)),
                        Container(
                          alignment: Alignment.topLeft,
                          padding: EdgeInsets.only(
                            top: 10,
                            left: 10,
                            right: 20,
                            //bottom: 10,
                          ),
                          child: Text(
                            cardAuthor,
                            style: TextStyle(
                              color: cardTextColor != null
                                  ? cardTextColor
                                  : Colors.black87,
                              fontSize: FontSize.FONT_17,
                              fontFamily: FontFamily.Gilroy,
                              fontWeight: FontWeight.bold,
                              letterSpacing: 1.0,
                              wordSpacing: 1.0,
                            ),
                            textAlign: TextAlign.center,
                            softWrap: true,
                            maxLines: 2,
                          ),
                        ),
                      ],
                    ),
                    Container(
                        padding: EdgeInsets.all(10),
                        child: new Text(
                          flag ? (firstHalf + "...") : (firstHalf + secondHalf),
                          style: TextStyle(
                            color: cardTextColor != null
                                ? cardTextColor
                                : Colors.black87,
                            fontSize: FontSize.FONT_15,
                            fontFamily: FontFamily.Gilroy,
                            fontWeight: FontWeight.w600,
                            letterSpacing: 1.0,
                            wordSpacing: 1.0,
                          ),
                        )),
                    Container(
                      padding: EdgeInsets.only(
                        //top: 10,
                        left: 10,
                        right: 20,
                        bottom: 10,
                      ),
                      child: new Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Container(
                            alignment: Alignment.center,
                            padding: EdgeInsets.only(
                              //   top: 10,
                              //left: 10,
                              right: 20,
                              bottom: 10,
                            ),
                            child: Text(
                              cardDate,
                              style: TextStyle(
                                color: cardTextColor != null
                                    ? cardTextColor
                                    : Colors.black87,
                                fontSize: FontSize.FONT_13,
                                fontFamily: FontFamily.Gilroy,
                                fontWeight: FontWeight.normal,
                                letterSpacing: 1.0,
                                wordSpacing: 1.0,
                              ),
                              textAlign: TextAlign.center,
                              softWrap: true,
                              maxLines: 2,
                            ),
                          ),
                          InkWell(
                            onTap: () {
                              onStateChange();
                            },
                            child: Container(
                              alignment: Alignment.center,
                              padding: EdgeInsets.only(
                                //   top: 10,
                                left: 10,
                                right: 20,
                                bottom: 10,
                              ),
                              child: Text(
                                flag ? "Show More" : "Show Less",
                                style: TextStyle(
                                  color: cardTextColor != null
                                      ? cardTextColor
                                      : Colors.black87,
                                  fontSize: FontSize.FONT_13,
                                  fontFamily: FontFamily.Gilroy,
                                  fontWeight: FontWeight.normal,
                                  letterSpacing: 1.0,
                                  wordSpacing: 1.0,
                                ),
                                textAlign: TextAlign.center,
                                softWrap: true,
                                maxLines: 2,
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
        ),
      );
    } else {
      return Container(
        width: width,
        height: height,
        child: Card(
          shape: shape,
          color: cardColor,
          child: Container(
            padding: EdgeInsets.all(padding),
            margin: EdgeInsets.all(padding),
            child: Text(
              cardText,
              style: TextStyle(letterSpacing: 1, color: cardTextColor),
            ),
          ),
          elevation: elevation,
        ),
      );
    }
  }

  void _setCardSize() {
    if (cardSize == CardSize.small) {
      padding ??= mini ? 5.0 : 6.0;
      width ??= 160;
      height ??= 100;
      fontSize = 15.0;
      imageSize = mini ? 34.0 : 40.0;
    } else if (cardSize == CardSize.medium) {
      padding ??= mini ? 5.5 : 6.5;
      width ??= 220;
      height ??= 150;
      fontSize = 17.0;
      imageSize = mini ? 40.0 : 46.0;
    } else {
      padding ??= mini ? 6.0 : 7.0;
      width ??= 250;
      height ??= 180;
      fontSize = 19.0;
      imageSize = mini ? 46.0 : 52.0;
    }
  }

  void _createStyle() {
    switch (cardType) {
      case CardType.customCard:
        cardText ??= 'Okay';
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;
      case CardType.cardWithTitle:
        cardTitle ??= "Title";
        cardName ??= "Name Harish";
        cardText ??= 'Okay';
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;

      case CardType.cardWithDashBoard:
        shape ??= RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        );
        cardTitle ??= "Harish";
        cardName ??= "Aagnia technologies";
        cardText ??= 'Okay';
        statusTag ??= '#tag';
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;

      case CardType.cardDashBoardDeal:
        shape ??= RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        );
        cardTitle ??= "Deals Created";
        cardPrice ??= "117";
        cardPercentage ??= '66';
        cardDate ??= "(Dec 2020)";
        trandValue ??= true;
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;

      case CardType.cardTask:
        shape ??= RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        );
        cardStatus ??= "Open";
        cardDesc ??= "Sharing the business proposel";
        cardDate ??= "05-05-2021";
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;

      case CardType.cardNotes:
        shape ??= RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        );
        cardAuthor ??= "Name";
        cardDesc ??=
            "They are ok with the solution, but need to review on budget";
        cardDate ??= "05-05-2021";
        cardTextColor ??= Colors.black87;
        cardColor ??= Colors.white;
        break;
    }
  }
}
