/*
    Refer License for Terms & Conditions
 */
import 'package:flutter/material.dart';

import 'package:flutter/cupertino.dart';

import 'Constants/AppColors.dart';
import 'Constants/ImageAssets.dart';
import 'Constants/AppStrings.dart';
import 'Constants/fontFamily.dart';
import 'package:bui_component/ButtonsComponent/buttonComponent.dart';

import 'Constants/fontSize.dart';
import 'Utils/SystemUtils.dart';

class ForgotScreenComponent extends StatefulWidget {
  final Color backgroundPrimaryColor;
  final Color buttonColor;
  final String title;
  final Function() sentFunction;
  final Function() signUpFunction;

  final TextEditingController emailController;
  final TextEditingController passwordController;
  final TextEditingController confirmPasswordController;

  final GlobalKey globalKey;

  final Widget imageLogo;
  final String logo;

  ForgotScreenComponent({
    Key key,
     this.backgroundPrimaryColor,
     this.buttonColor,
     this.title,
     this.sentFunction,
     this.signUpFunction,
     this.emailController,
     this.passwordController,
     this.confirmPasswordController,
     this.globalKey,
    this.imageLogo,
    this.logo,
  }) : super(key: key);

  @override
  _ForgotScreenComponentState createState() => _ForgotScreenComponentState();
}

class _ForgotScreenComponentState extends State<ForgotScreenComponent> {
  bool _obscureText = false;
  bool _passwordVisible = false;

  void _toggle() {
    setState(() {
      _obscureText = !_obscureText;
      _passwordVisible = !_passwordVisible;
    });
  }

  bool _confirmObscureText = false;
  bool _confirmPasswordVisible = false;

  void _confirmToggle() {
    setState(() {
      _confirmObscureText = !_confirmObscureText;
      _confirmPasswordVisible = !_confirmPasswordVisible;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: widget.backgroundPrimaryColor != null
          ? widget.backgroundPrimaryColor
          : AppColors.primaryColor,
      body: ListView(
        children: <Widget>[
          Container(
            padding: EdgeInsets.only(
                top: 60.0, left: 30.0, right: 30.0, bottom: 30.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.center,
              children: <Widget>[
                Center(
                    child: widget.imageLogo ??
                        Image.asset(
                          widget.logo ?? ImageAssets.FACEBOOK,
                          height: MediaQuery.of(context).size.height * 0.2,
                        )),
              ],
            ),
          ),
          _forgotWidget,
          //mailSentMessgaeShow,
          //resetPasswordWidget
        ],
      ),
    );
  }

  Widget get _forgotWidget {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Form(
          key: widget.globalKey,
          child: Column(
            children: [
              Container(
                alignment: Alignment.center,
                padding: EdgeInsets.all(20),
                child: Text(
                  AppStrings.FORGOT_PASSWORD_TITLE,
                  style: TextStyle(
                    fontFamily: FontFamily.Gilroy,
                    fontSize: FontSize.FONT_30,
                    color: widget.backgroundPrimaryColor != null
                        ? widget.backgroundPrimaryColor
                        : AppColors.primaryColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              Container(
                child: Text(
                  AppStrings.FORGOT_PASSWORD_RESET_TEXT,
                  style: TextStyle(
                    fontFamily: FontFamily.Poppins,
                    fontSize: FontSize.FONT_13,
                    color: AppColors.APP_COLOR_PRIMARY,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              SizedBox(
                height: 20,
              ),
              TextFormField(
                controller: widget.emailController,
                decoration: new InputDecoration(
                  hintText: AppStrings.EMAIL,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.backgroundPrimaryColor != null
                          ? widget.backgroundPrimaryColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                  errorBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (val.length == 0) {
                    return AppStrings.EMAIL_CANNOT_BE_EMPTY;
                  } else if (!SystemUtils.hasMatchEmail(val)) {
                    return AppStrings.INVALID_EMAIL;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.emailAddress,
                style: new TextStyle(
                  fontFamily: FontFamily.Poppins,
                ),
              ),
              SizedBox(
                height: 20,
              ),
              Button.mini(
                btnColor: widget.backgroundPrimaryColor != null
                    ? widget.backgroundPrimaryColor
                    : AppColors.primaryColor,
                onPressed: () {
                  widget.sentFunction();
                },
                buttonType: ButtonType.custom,
                btnText: AppStrings.SENT,
                btnTextColor: AppColors.APP_COLOR_WHITE,
              ),
              SizedBox(
                height: 20,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Container(
                    child: Text(
                      AppStrings.CREATE_AN_ACCOUNT,
                      style: TextStyle(
                          fontFamily: FontFamily.Gilroy,
                          fontSize: FontSize.FONT_13,
                          color: widget.backgroundPrimaryColor != null
                              ? widget.backgroundPrimaryColor
                              : AppColors.primaryColor,
                          fontWeight: FontWeight.normal),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  InkWell(
                    onTap: () {
                      widget.signUpFunction();
                    },
                    child: Container(
                      child: Text(
                        AppStrings.SIGN_UP,
                        style: TextStyle(
                            fontFamily: FontFamily.Gilroy,
                            fontSize: FontSize.FONT_15,
                            color: widget.backgroundPrimaryColor != null
                                ? widget.backgroundPrimaryColor
                                : AppColors.primaryColor,
                            fontWeight: FontWeight.bold),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 20,
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget get mailSentMessgaeShow {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Column(
          children: [
            Container(
              alignment: Alignment.center,
              padding: EdgeInsets.all(20),
              child: Text(
                AppStrings.FORGOT_PASSWORD_TITLE,
                style: TextStyle(
                  fontFamily: FontFamily.Gilroy,
                  fontSize: FontSize.FONT_30,
                  color: widget.backgroundPrimaryColor != null
                      ? widget.backgroundPrimaryColor
                      : AppColors.primaryColor,
                ),
                textAlign: TextAlign.center,
              ),
            ),
            SizedBox(
              height: 50,
            ),
            Container(
              child: Text(
                AppStrings.FORGOT_PASSWORD_RESET_TEXT,
                style: TextStyle(
                  fontFamily: FontFamily.Poppins,
                  fontSize: FontSize.FONT_13,
                  color: AppColors.APP_COLOR_PRIMARY,
                ),
                textAlign: TextAlign.center,
              ),
            ),
            SizedBox(
              height: 50,
            ),
            Row(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Container(
                  child: Text(
                    AppStrings.CREATE_AN_ACCOUNT,
                    style: TextStyle(
                        fontFamily: FontFamily.Gilroy,
                        fontSize: FontSize.FONT_13,
                        color: widget.backgroundPrimaryColor != null
                            ? widget.backgroundPrimaryColor
                            : AppColors.primaryColor,
                        fontWeight: FontWeight.normal),
                    textAlign: TextAlign.center,
                  ),
                ),
                InkWell(
                  onTap: () {
                    widget.signUpFunction();
                  },
                  child: Container(
                    child: Text(
                      AppStrings.SIGN_UP,
                      style: TextStyle(
                          fontFamily: FontFamily.Gilroy,
                          fontSize: FontSize.FONT_15,
                          color: widget.backgroundPrimaryColor != null
                              ? widget.backgroundPrimaryColor
                              : AppColors.primaryColor,
                          fontWeight: FontWeight.bold),
                      textAlign: TextAlign.center,
                    ),
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }

  Widget get resetPasswordWidget {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Form(
          key: widget.globalKey,
          child: Column(
            children: [
              Container(
                alignment: Alignment.center,
                padding: EdgeInsets.all(20),
                child: Text(
                  AppStrings.RESET_PASSWORD,
                  style: TextStyle(
                    fontFamily: FontFamily.Gilroy,
                    fontSize: FontSize.FONT_30,
                    color: widget.backgroundPrimaryColor != null
                        ? widget.backgroundPrimaryColor
                        : AppColors.primaryColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              TextFormField(
                controller: widget.passwordController,
                obscureText: !_obscureText,
                decoration: new InputDecoration(
                  suffixIcon: GestureDetector(
                    onTap: () {
                      _toggle();
                    },
                    child: Icon(
                      _passwordVisible
                          ? Icons.visibility
                          : Icons.visibility_off,
                      color: !_passwordVisible
                          ? AppColors.halfWhite
                          : AppColors.textColor,
                    ),
                  ),
                  hintText: AppStrings.NEW_PASSWORD,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.backgroundPrimaryColor != null
                          ? widget.backgroundPrimaryColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                  errorBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (val.length == 0) {
                    return AppStrings.PASSWORD_CANNOT_BE_EMPTY;
                  } else if (val.length < 6) {
                    return AppStrings.PASSWORD_TOO_SHORT;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.text,
                style: new TextStyle(
                  fontFamily: FontFamily.Gilroy,
                ),
              ),
              SizedBox(
                height: 20,
              ),
              TextFormField(
                controller: widget.confirmPasswordController,
                obscureText: !_confirmObscureText,
                decoration: new InputDecoration(
                  suffixIcon: GestureDetector(
                    onTap: () {
                      _confirmToggle();
                    },
                    child: Icon(
                      _confirmPasswordVisible
                          ? Icons.visibility
                          : Icons.visibility_off,
                      color: !_passwordVisible
                          ? AppColors.halfWhite
                          : AppColors.textColor,
                    ),
                  ),
                  hintText: AppStrings.CONFIRM_PASSWORD,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.backgroundPrimaryColor != null
                          ? widget.backgroundPrimaryColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                  errorBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                  border: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.errorWarning,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (val.length == 0) {
                    return AppStrings.PASSWORD_CANNOT_BE_EMPTY;
                  } else if (val.length < 6) {
                    return AppStrings.PASSWORD_TOO_SHORT;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.text,
                style: new TextStyle(
                  fontFamily: FontFamily.Gilroy,
                ),
              ),
              SizedBox(
                height: 20,
              ),
              Button.mini(
                btnColor: widget.backgroundPrimaryColor != null
                    ? widget.backgroundPrimaryColor
                    : AppColors.primaryColor,
                onPressed: () {
                  widget.sentFunction();
                },
                buttonType: ButtonType.custom,
                btnText: AppStrings.SENT,
                btnTextColor: AppColors.APP_COLOR_WHITE,
              ),
              SizedBox(
                height: 20,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Container(
                    child: Text(
                      AppStrings.CREATE_AN_ACCOUNT,
                      style: TextStyle(
                          fontFamily: FontFamily.Gilroy,
                          fontSize: FontSize.FONT_13,
                          color: widget.backgroundPrimaryColor != null
                              ? widget.backgroundPrimaryColor
                              : AppColors.primaryColor,
                          fontWeight: FontWeight.normal),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  InkWell(
                    onTap: () {
                      widget.signUpFunction();
                    },
                    child: Container(
                      child: Text(
                        AppStrings.SIGN_UP,
                        style: TextStyle(
                            fontFamily: FontFamily.Gilroy,
                            fontSize: FontSize.FONT_15,
                            color: widget.backgroundPrimaryColor != null
                                ? widget.backgroundPrimaryColor
                                : AppColors.primaryColor,
                            fontWeight: FontWeight.bold),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 20,
              ),
            ],
          ),
        ),
      ),
    );
  }
}
