/*
    Refer License for Terms & Conditions
 */

import 'dart:convert';

import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';

// ignore: must_be_immutable
class JsonSchema extends StatefulWidget {
  final Map initValue;
  Color theme;
  Color secondaryColorTheme;
  String buttonText;

  JsonSchema(
      {this.form,
      this.onChanged,
      this.calendarColor,
      this.theme,
      this.padding,
      this.formMap,
      this.errorMessages = const {},
      this.validations = const {},
      this.decorations = const {},
      this.buttonSave,
      this.actionCancel,
      this.buttonCancel,
      this.actionSave,
      this.buttonText,
      this.secondaryColorTheme,
      this.initValue});

  Color calendarColor;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final String form;
  final Map formMap;
  final double padding;
  final bool buttonSave;
  final bool buttonCancel;
  final Function actionSave;
  final Function actionCancel;
  final ValueChanged<dynamic> onChanged;

  @override
  _CoreFormState createState() =>
      new _CoreFormState(formMap ?? json.decode(form));
}

class _CoreFormState extends State<JsonSchema> {
  final dynamic formGeneral;

  int radioValue;
  Map initValue;
  Map<String, String> _datevalueMap = {};

  @override
  void initState() {
    initValue = widget.initValue;

    super.initState();
  }

  // validators

  String isRequired(item, value) {
    if (value.isEmpty) {
      return widget.errorMessages[item['key']] ?? 'Required This Field';
    }
    return null;
  }

  String validateEmail(item, String value) {
    String p = "[a-zA-Z0-9\+\.\_\%\-\+]{1,256}" +
        "\\@" +
        "[a-zA-Z0-9][a-zA-Z0-9\\-]{0,64}" +
        "(" +
        "\\." +
        "[a-zA-Z0-9][a-zA-Z0-9\\-]{0,25}" +
        ")+";
    RegExp regExp = new RegExp(p);

    if (regExp.hasMatch(value)) {
      return null;
    }
    return 'Email is not valid';
  }

  bool labelHidden(item) {
    if (item.containsKey('hiddenLabel')) {
      if (item['hiddenLabel'] is bool) {
        return !item['hiddenLabel'];
      }
    } else {
      return true;
    }
    return false;
  }

  // Return widgets

  List<Widget> jsonToForm() {
    List<Widget> listWidget = new List<Widget>();
    if (formGeneral['title'] != null) {
      listWidget.add(Text(
        formGeneral['title'],
        style: new TextStyle(
            letterSpacing: 1,
            fontWeight: FontWeight.normal,
            fontSize: FontSize.FONT_20,
            color: AppColors.textColor),
      ));
    }
    if (formGeneral['description'] != null) {
      listWidget.add(Text(
        formGeneral['description'],
        style: new TextStyle(
            letterSpacing: 1,
            fontSize: FontSize.FONT_14,
            fontStyle: FontStyle.italic),
      ));
    }

    for (var count = 0; count < formGeneral['fields'].length; count++) {
      Map item = formGeneral['fields'][count];

      if (item['type'] == "input" ||
          item['type'] == "password" ||
          item['type'] == "email" ||
          item['type'] == "textArea" ||
          item['type'] == "date" ||
          item['type'] == "textInput") {
        Widget label = SizedBox.shrink();

        if (labelHidden(item)) {
          label = Visibility(
            visible: item['visibility'] ?? true,
            child: new Container(
                padding: EdgeInsets.only(bottom: 5, top: 5),
                child: RichText(
                  text: TextSpan(
                    text: item['label'],
                    style: new TextStyle(
                        letterSpacing: 1,
                        fontWeight: FontWeight.normal,
                        fontSize: FontSize.FONT_13,
                        color: AppColors.textColor,
                        fontFamily: FontFamily.Gilroy),
                    children: <TextSpan>[
                      //  item['label'].toString().endsWith('*')
                      item['required'] == true
                          ? TextSpan(
                              text: ' *',
                              style: TextStyle(
                                  letterSpacing: 1,
                                  fontWeight: FontWeight.bold,
                                  color: AppColors.errorWarning))
                          : TextSpan(
                              text: '',
                              style: TextStyle(
                                  letterSpacing: 1,
                                  fontWeight: FontWeight.bold,
                                  fontSize: FontSize.FONT_13,
                                  fontFamily: FontFamily.Gilroy,
                                  color: AppColors.errorWarning)),
                    ],
                  ),
                )),
          );
        }
        if (item[label].toString() != 'date')
          listWidget.add(new Container(
            //margin: new EdgeInsets.only(left: 10,right: 10,bottom: 10),

            child: new Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Container(
                    padding: new EdgeInsets.only(
                      left: 5,
                      right: 5,
                    ),
                    child: label),
                item['type'].toString() != 'date'
                    ? Visibility(
                        visible: item["visibility"] ?? true,
                        child: Container(
                          padding: new EdgeInsets.only(
                              left: 10, right: 10, bottom: 1),
                          child: new TextFormField(
                            controller: null,
                            style: TextStyle(
                                letterSpacing: 1,
                                fontSize: FontSize.FONT_15,
                                fontFamily: FontFamily.Gilroy,
//                              fontWeight: FontWeight.bold,
                                color: AppColors.APP_COLOR_BLACK),
                            readOnly: item['readOnly'] == true ||
                                    item['readOnly'] == 'True' ||
                                    item['readOnly'] == 'true'
                                ? true
                                : false,
                            initialValue:
                                formGeneral['fields'][count]['value'] ?? null,
                            decoration: item['decoration'] ??
                                widget.decorations[item['key']] ??
                                new InputDecoration(
                                  hintText: item['placeholder'] ?? "",
                                  helperText: item['helpText'] ?? "",
                                  hintStyle: TextStyle(
                                      letterSpacing: 1,
                                      fontFamily: FontFamily.Gilroy),
                                  fillColor: AppColors.appWhite,
                                  focusedBorder: OutlineInputBorder(
                                    borderRadius: BorderRadius.circular(7.0),
                                    borderSide: BorderSide(
                                        color:
                                            widget.theme ?? AppColors.halfWhite,
                                        width: 2),
                                  ),
                                  enabledBorder: OutlineInputBorder(
                                    borderRadius: BorderRadius.circular(7.0),
                                    borderSide: BorderSide(
                                      color: AppColors.halfWhite,
                                      width: 2.0,
                                    ),
                                  ),
                                  errorBorder: OutlineInputBorder(
                                    borderRadius: BorderRadius.circular(7.0),
                                    borderSide: BorderSide(
                                      color: AppColors.errorWarning,
                                      width: 2.0,
                                    ),
                                  ),
                                  border: OutlineInputBorder(
                                    borderRadius: BorderRadius.circular(7.0),
                                    borderSide: BorderSide(
                                      color: AppColors.errorWarning,
                                      width: 2.0,
                                    ),
                                  ),
                                ),
                            maxLines: item['type'] == "textArea" ? 10 : 1,
                            keyboardType: item['keyboardType'] == "number"
                                ? TextInputType.number
                                : item['keyboardType'] == "email"
                                    ? TextInputType.emailAddress
                                    : item['keyboardType'] == "number"
                                        ? TextInputType.number
                                        : item['keyboardType'] == "phone"
                                            ? TextInputType.phone
                                            : TextInputType.text,
                            maxLength: item['type'] == "textArea"
                                ? item['textLength'] ?? 1000
                                : null,
                            onChanged: (String value) {
                              formGeneral['fields'][count]['value'] = value;
                              _handleChanged();
                            },
                            obscureText:
                                item['type'] == "password" ? true : false,
                            validator: (value) {
                              if (widget.validations.containsKey(item['key'])) {
                                return widget.validations[item['key']](
                                    item, value);
                              }
                              if (item.containsKey('validator')) {
                                if (item['validator'] != null) {
                                  if (item['validator'] is Function) {
                                    return item['validator'](item, value);
                                  }
                                }
                              }
                              if (item['type'] == "email") {
                                return validateEmail(item, value);
                              }

                              if (item.containsKey('required')) {
                                if (item['required'] == true ||
                                    item['required'] == 'True' ||
                                    item['required'] == 'true') {
                                  return isRequired(item, value);
                                }
                              }

                              return null;
                            },
                          ),
                        ),
                      )
                    : Container(),
              ],
            ),
          ));
      }
      if (item['type'] == "radioButton") {
        List<Widget> radios = [];

        if (labelHidden(item)) {
          radios.add(new Text(item['label'],
              style: new TextStyle(
                  letterSpacing: 1,
                  fontWeight: FontWeight.bold,
                  fontSize: 16.0)));
        }
        radioValue = item['value'];
        for (var i = 0; i < item['items'].length; i++) {
          radios.add(
            new Row(
              children: <Widget>[
                new Expanded(
                    child: new Text(
                        formGeneral['fields'][count]['items'][i]['label'])),
                new Radio<int>(
                    value: formGeneral['fields'][count]['items'][i]['value'],
                    groupValue: radioValue,
                    onChanged: (int value) {
                      this.setState(() {
                        radioValue = value;
                        formGeneral['fields'][count]['value'] = value;
                        _handleChanged();
                      });
                    })
              ],
            ),
          );
        }

        listWidget.add(
          new Container(
            margin: new EdgeInsets.only(top: 5.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: radios,
            ),
          ),
        );
      }

      if (item['type'] == "switch") {
        if (item['value'] == null) {
          formGeneral['fields'][count]['value'] = false;
        }
        listWidget.add(
          new Container(
            margin: new EdgeInsets.only(top: 5.0),
            child: new Row(children: <Widget>[
              new Expanded(child: new Text(item['label'])),
              new Switch(
                value: item['value'] ?? false,
                onChanged: (bool value) {
                  this.setState(() {
                    formGeneral['fields'][count]['value'] = value;
                    _handleChanged();
                  });
                },
              ),
            ]),
          ),
        );
      }

      if (item['type'] == 'date') {
        Future _selectDate() async {
          DateTime picked = await showDatePicker(
            context: context,
            initialDate: DateTime.now(),
            firstDate: DateTime(1880),
            lastDate: DateTime(2100),
            helpText: item['label'] ?? 'Select  Date',
            cancelText: item['cancelText'] ?? 'Cancel',
            confirmText: item['confirmText'] ?? 'Okay',
            fieldLabelText: item['fieldLabelText'] ?? 'Select Date',
            fieldHintText: item['fieldHintText'] ?? 'Month/Date/Year',
            errorFormatText: item['errorFormatText'] ?? 'Enter valid date',
            errorInvalidText:
                item['errorInvalidText'] ?? 'Enter date in valid range',
            builder: (BuildContext context, Widget child) {
              return Theme(
                data: Theme.of(context).copyWith(
                    colorScheme: Theme.of(context).colorScheme.copyWith(
                        primary: widget.calendarColor ??
                            AppColors.APP_COLOR_PRIMARY)),
                child: child,
              );
            },
          );
          if (picked != null) {
            /*setState(() => _datevalueMap[item["title"]] =
                picked.toString().substring(0, 10));
            */
            setState(() {
              _datevalueMap[item["title"]] = picked.toString().substring(0, 10);
              print("========" + picked.toString().substring(0, 10));
            });
          }
        }

        listWidget.add(
          Visibility(
            visible: item["visibility"] ?? true,
            child: Container(
                //margin: EdgeInsets.symmetric(vertical: 10.0),
                padding: new EdgeInsets.only(left: 10, right: 10, bottom: 1),
                child: TextFormField(
                  //initialValue:
                  // _datevalueMap != null ? _datevalueMap[item["title"]] : null,
                  autofocus: false,
                  controller:
                      TextEditingController(text: _datevalueMap[item["title"]]),
                  validator: (String value) {
                    if (value.isEmpty) {
                      return 'Please select a date';
                    }
                    return null;
                  },
                  onChanged: (String value) {
                    _handleChanged();
                  },
                  onTap: () async {
                    await _selectDate();
                    formGeneral[item["title"]] = _datevalueMap[item["title"]];
                    _handleChanged();
                  },
                  /*decoration: InputDecoration(
                    contentPadding: EdgeInsets.fromLTRB(10, 0, 10, 0),
                   // labelText: item["label"],
                    border: OutlineInputBorder(
                        borderRadius: BorderRadius.circular(5.0)),
                    suffixIcon: Icon(
                      Icons.calendar_today,
                    ),
                  ),*/
                  decoration: item['decoration'] ??
                      widget.decorations[item['key']] ??
                      new InputDecoration(
                        hintText: item['placeholder'] ?? "",
                        helperText: item['helpText'] ?? "",
                        prefixIcon: Icon(
                          Icons.calendar_today_rounded,
                          color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                        ),
                        hintStyle: TextStyle(
                            letterSpacing: 1, fontFamily: FontFamily.Gilroy),
                        fillColor: AppColors.appWhite,
                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                              color: widget.theme ?? AppColors.halfWhite,
                              width: 2),
                        ),
                        enabledBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: AppColors.halfWhite,
                            width: 2.0,
                          ),
                        ),
                        errorBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: AppColors.errorWarning,
                            width: 2.0,
                          ),
                        ),
                        border: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: AppColors.errorWarning,
                            width: 2.0,
                          ),
                        ),
                      ),
                )),
          ),
        );
      }

      if (item['type'] == "checkbox") {
        List<Widget> checkboxes = [];
        if (labelHidden(item)) {
          checkboxes.add(new Text(item['label'],
              style: new TextStyle(
                  letterSpacing: 1,
                  fontWeight: FontWeight.bold,
                  fontSize: 16.0)));
        }
        for (var i = 0; i < item['items'].length; i++) {
          checkboxes.add(
            new Row(
              children: <Widget>[
                new Expanded(
                    child: new Text(
                        formGeneral['fields'][count]['items'][i]['label'])),
                new Checkbox(
                  value: formGeneral['fields'][count]['items'][i]['value'],
                  onChanged: (bool value) {
                    this.setState(
                      () {
                        formGeneral['fields'][count]['items'][i]['value'] =
                            value;
                        _handleChanged();
                      },
                    );
                  },
                ),
              ],
            ),
          );
        }

        listWidget.add(
          new Container(
            margin: new EdgeInsets.only(top: 5.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: checkboxes,
            ),
          ),
        );
      }

      if (item['type'] == "select") {
        Widget label = SizedBox.shrink();
        if (labelHidden(item)) {
          /*label = Container(
            padding: EdgeInsets.only(bottom: 5, top: 5,left: 5),
            child: new Text(item['label'],
                style: new TextStyle(
                          letterSpacing: 1,
                    fontWeight: FontWeight.normal,
                    fontSize: FontSize.FONT_13,
                    color: AppColors.textColor,
                    fontFamily: FontFamily.Gilroy)),
          );*/
          label = new Container(
              padding: EdgeInsets.only(bottom: 5, top: 5, left: 5),
              child: RichText(
                text: TextSpan(
                  text: item['label'],
                  style: new TextStyle(
                      letterSpacing: 1,
                      fontWeight: FontWeight.normal,
                      fontSize: FontSize.FONT_13,
                      color: AppColors.textColor,
                      fontFamily: FontFamily.Gilroy),
                  children: <TextSpan>[
                    //  item['label'].toString().endsWith('*')
                    item['required'] == true
                        ? TextSpan(
                            text: ' *',
                            style: TextStyle(
                                letterSpacing: 1,
                                fontWeight: FontWeight.bold,
                                color: AppColors.errorWarning))
                        : TextSpan(
                            text: '',
                            style: TextStyle(
                                letterSpacing: 1,
                                fontWeight: FontWeight.bold,
                                fontSize: FontSize.FONT_13,
                                fontFamily: FontFamily.Gilroy,
                                color: AppColors.errorWarning)),
                  ],
                ),
              ));
        }

        listWidget.add(new Container(
          margin: new EdgeInsets.only(top: 5.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              label,
              new Container(
                margin: new EdgeInsets.only(left: 10.0, right: 10),
                decoration: ShapeDecoration(
                  color: formGeneral['fields'][count]['readOnly'] == true
                      ? AppColors.halfWhite
                      : Colors.white,
                  shape: RoundedRectangleBorder(
                    side: BorderSide(
                        width: 2.0,
                        style: BorderStyle.solid,
                        color: AppColors.APP_GRAY_1),
                    borderRadius: BorderRadius.all(Radius.circular(5.0)),
                  ),
                ),
                width: MediaQuery.of(context).size.width,
                padding: EdgeInsets.all(5),
                child: IgnorePointer(
                  ignoring: formGeneral['fields'][count]['readOnly'] != null
                      ? formGeneral['fields'][count]['readOnly']
                      : false,
                  child: DropdownButton<String>(
                    underline: Container(),
                    isExpanded: true,
                    hint: item['hint'] ?? new Text(""),
                    value: formGeneral['fields'][count]['value'],
                    onChanged: (String newValue) {
                      setState(() {
                        /*if (formGeneral['fields'][count]['readOnly'] == true ||
                            formGeneral['fields'][count]['readOnly'] == 'true' ||
                            formGeneral['fields'][count]['readOnly'] == 'True') {

                          print("Rest=====");
                        } else {
                          print("Rest1=====");
                          formGeneral['fields'][count]['value'] = newValue;
                          _handleChanged();
                        }*/
                        formGeneral['fields'][count]['value'] = newValue;
                        _handleChanged();
                      });
                    },
                    items: item['items']
                        .map<DropdownMenuItem<String>>((dynamic data) {
                      return DropdownMenuItem<String>(
                        value: data['value'],
                        child: new Text(
                          data['label'],
                          style: new TextStyle(
                            letterSpacing: 1,
                            color: Colors.black,
                            fontWeight: FontWeight.normal,
                            fontSize: FontSize.FONT_13,
                            fontFamily: FontFamily.Gilroy,
                          ),
                        ),
                      );
                    }).toList(),
                  ),
                ),
              ),
            ],
          ),
        ));
      }
    }

    if (widget.buttonSave) {
      listWidget.add(Container(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            /*  new Container(
              width: MediaQuery.of(context).size.width * 0.4,
              margin: EdgeInsets.only(top: 10.0),
              child: InkWell(
                onTap: () {
                  widget.actionCancel();
                },
                child: widget.buttonCancel,
              ),
            ),*/

            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Button.mini(
                width: MediaQuery.of(context).size.width * 0.3,
                onPressed: () {
                  widget.actionCancel();
                },
                buttonSize: ButtonSize.medium,
                buttonType: ButtonType.cancel,
                btnColor: AppColors.APP_BUTTON_GREY,
                btnTextColor:
                    widget.secondaryColorTheme ?? AppColors.APP_COLOR_PRIMARY,
                elevation: 10,
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Button.mini(
                width: MediaQuery.of(context).size.width * 0.3,
                onPressed: () {
                  if (_formKey.currentState.validate()) {
                    widget.actionSave(formGeneral);
                  }
                },
                buttonSize: ButtonSize.medium,
                buttonType: ButtonType.custom,
                btnColor:
                    widget.secondaryColorTheme ?? AppColors.APP_COLOR_PRIMARY,
                btnTextColor: Colors.white,
                btnText: widget.buttonText ?? "Save",
                elevation: 10,
              ),
            ),
          ],
        ),
      ));
    }
    return listWidget;
  }

  _CoreFormState(this.formGeneral);

  void _handleChanged() {
    widget.onChanged(formGeneral);
  }

  final _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    return Form(
      //autovalidate: formGeneral['autoValidated'] ?? false,
      key: _formKey,
      child: new Container(
        padding: new EdgeInsets.all(widget.padding ?? 8.0),
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: jsonToForm(),
        ),
      ),
    );
  }
}
