/*
    Refer License for Terms & Conditions
 */
import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/Constants/ImageAssets.dart';
import 'package:country_code_picker/country_code_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'ButtonsComponent/button_builder.dart';
import 'Constants/AppColors.dart';
import 'Constants/AppStrings.dart';
import 'Constants/fontFamily.dart';
import 'Constants/fontSize.dart';
import 'Utils/SystemUtils.dart';

class SignUpScreenComponent extends StatefulWidget {
  final Function() facebookSignUp;
  final Function() googleSignUp;
  final Function() phoneNumberSignInFunction;
  final Function() emailPasswordSignInFunction;
  final Function() signInFunction;
  final Widget imageLogo;
  final Function(CountryCode countryCode) countryCodePicker;
  final TextEditingController phoneNumberController;
  final TextEditingController emailController;
  final TextEditingController passwordController;
  final TextEditingController firstNameController;
  final TextEditingController lastNameController;
  final GlobalKey globalKey;
  final GlobalKey phoneGlobalKey;
  Color themeColor;
  final Color buttonColor;
  final Color containerPrimaryColor;
  final String logo;
  final List<String> tabEnabledName;
  String countryCode;
  String initalSelection;

  SignUpScreenComponent({
    Key key,
     this.facebookSignUp,
     this.googleSignUp,
     this.phoneNumberSignInFunction,
     this.countryCodePicker,
     this.phoneNumberController,
     this.globalKey,
     this.emailPasswordSignInFunction,
     this.signInFunction,
     this.emailController,
     this.passwordController,
     this.firstNameController,
     this.lastNameController,
     this.phoneGlobalKey,
     this.themeColor,
     this.buttonColor,
     this.containerPrimaryColor,
    this.logo,
     this.tabEnabledName,
    this.countryCode,
    this.initalSelection,
    this.imageLogo,
  }) : super(key: key);

  @override
  _SignUpScreenComponentState createState() => _SignUpScreenComponentState();
}

class _SignUpScreenComponentState extends State<SignUpScreenComponent> {
  bool _obscureText = false;
  bool _passwordVisible = false;

  String phoneNumber = "";

  String initalSelection = 'IN';


  void _toggle() {
    setState(() {
      _obscureText = !_obscureText;
      _passwordVisible = !_passwordVisible;
    });
  }

  @override
  void initState() {
    super.initState();

    getStringValuesSF();
  }

  getStringValuesSF() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    //Return int
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        widget.themeColor = Color(colorCodeInt);
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: widget.themeColor != null
            ? widget.themeColor
            : AppColors.primaryColor,
        body: ListView(
          children: [
            Container(
              padding: EdgeInsets.only(
                  top: 60.0, left: 30.0, right: 30.0, bottom: 30.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                children: <Widget>[
                  Center(
                      child: widget.imageLogo ??
                          Image.asset(
                            widget.logo ?? ImageAssets.FACEBOOK,
                            height: MediaQuery.of(context).size.height * 0.2,
                          )),
                ],
              ),
            ),
            DefaultTabController(
                length: widget.tabEnabledName.length,
                child: Column(
                  children: [
                    TabBar(
                      indicatorColor: AppColors.APP_COLOR_WHITE,
                      unselectedLabelColor: AppColors.APP_GRAY_1,
                      labelColor: AppColors.APP_COLOR_WHITE,
                      isScrollable: true,
                      physics: NeverScrollableScrollPhysics(),
                      onTap: (i) {
                        SystemUtils.hideKeyboard(context);
                      },
                      tabs: List<Widget>.generate(widget.tabEnabledName.length,
                          (int index) {
                        return Container(
                            width: MediaQuery.of(context).size.width * 0.25,
                            child: new Tab(
                              text: widget.tabEnabledName[index],
                            ));
                      }),
                    ),
                    Container(
                      width: MediaQuery.of(context).size.width,
                      height: MediaQuery.of(context).size.height,
                      child: TabBarView(
                        physics: NeverScrollableScrollPhysics(),
                        children: List<Widget>.generate(
                            widget.tabEnabledName.length, (int index) {
                          if (widget.tabEnabledName[index] ==
                              AppStrings.SOCIAL) {
                            return _socialWidget;
                          } else if (widget.tabEnabledName[index] ==
                              AppStrings.MOBILE) {
                            return _mobileWidget;
                          } else if (widget.tabEnabledName[index] ==
                              AppStrings.EMAIL) {
                            return _emailWidget;
                          } else {
                            return Container();
                          }
                        }),
                      ),
                    ),
                  ],
                )),
          ],
        ));
  }

  Widget get _socialWidget {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Column(
          children: [
            Container(
              padding: EdgeInsets.all(20),
              child: Text(
                AppStrings.SOCIAL_LOGIN,
                style: TextStyle(
                  fontFamily: FontFamily.Gilroy,
                  fontSize: FontSize.FONT_20,
                  color: widget.themeColor ?? AppColors.APP_COLOR_PRIMARY,
                ),
                textAlign: TextAlign.center,
              ),
            ),
            SizedBox(
              width: 40,
              height: 10,
            ),
            Container(
              padding: EdgeInsets.all(20),
              child: Text(
                AppStrings.QUICK_SIGN_IN_SOCIAL,
                style: TextStyle(
                  fontFamily: FontFamily.Gilroy,
                  fontSize: FontSize.FONT_13,
                  color: AppColors.APP_GRAY_1,
                ),
                textAlign: TextAlign.center,
              ),
            ),
            SizedBox(
              width: 40,
              height: 10,
            ),
            Button.mini(
              btnColor: AppColors.APP_COLOR_WHITE,
              width: MediaQuery.of(context).size.width * 0.6,
              onPressed: () {
                widget.googleSignUp();
              },
              elevation: 20,
              buttonType: ButtonType.google,
              btnText: AppStrings.SIGN_IN_WITH_GOOGLE,
              btnTextColor: widget.themeColor ?? AppColors.APP_COLOR_WHITE,
            ),
            /*  InkWell(
              onTap: () {
                widget.googleSignUp();
              },
              child: Card(
                elevation: 10,
                child: Container(
                  width: MediaQuery.of(context).size.width * 0.6,
                  height: 48,
                  padding: EdgeInsets.all(6),
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      SvgPicture.asset(
                        SvgPictures.GOOGLE_SVG,
                        height: 40,
                        width: 40,
                      ),
                      SizedBox(width: 12),
                      Text(
                        AppStrings.SIGN_IN_WITH_GOOGLE,
                        style: TextStyle(
                            fontSize: FontSize.FONT_13,
                            fontWeight: FontWeight.bold,
                            fontFamily: FontFamily.Gilroy,
                            color: AppColors.APP_COLOR_PRIMARY),
                      ),
                    ],
                  ),
                ),
              ),
            ),*/
            SizedBox(
              width: 30,
              height: 10,
            ),
            Button.mini(
              btnColor: AppColors.FACEBOOK_COLOR,
              width: MediaQuery.of(context).size.width * 0.6,
              onPressed: () {
                widget.facebookSignUp();
              },
              elevation: 20,
              buttonSize: ButtonSize.small,
              buttonType: ButtonType.facebook,
              btnText: AppStrings.SIGN_IN_WITH_FACEBOOK,
              btnTextColor: AppColors.APP_COLOR_WHITE,
            ),

            /*     InkWell(
              onTap: () {
                widget.facebookSignUp();
              },
              child: Card(
                color: AppColors.FACEBOOK_COLOR,
                elevation: 10,
                child: Container(
                  width: MediaQuery.of(context).size.width * 0.6,
                  height: 48,
                  padding: EdgeInsets.all(6),
                  child: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      SvgPicture.asset(
                        SvgPictures.FB_SVG,
                        height: 40,
                        width: 40,
                      ),
                      SizedBox(width: 12),
                      Text(
                        AppStrings.SIGN_IN_WITH_FACEBOOK,
                        style: TextStyle(
                            fontSize: FontSize.FONT_13,
                            fontWeight: FontWeight.bold,
                            fontFamily: FontFamily.Gilroy,
                            color: AppColors.APP_COLOR_WHITE),
                      ),
                    ],
                  ),
                ),
              ),
            ),*/
          ],
        ),
      ),
    );
  }

  Widget get _mobileWidget {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Form(
          key: widget.phoneGlobalKey,
          child: Column(
            children: [
              Container(
                padding: EdgeInsets.all(20),
                child: Text(
                  AppStrings.SIGN_IN,
                  style: TextStyle(
                    fontFamily: FontFamily.Gilroy,
                    fontSize: FontSize.FONT_20,
                    color: widget.themeColor != null
                        ? widget.themeColor
                        : AppColors.primaryColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              TextFormField(
                autofocus: true,
                onChanged: (val) {
                  setState(() {
                    if (val.length == 10) {
                      SystemUtils.hideKeyboard(context);
                    }
                  });
                },
                controller: widget.phoneNumberController,
                decoration: new InputDecoration(
                  prefix: Container(
                    margin: EdgeInsets.only(right: 20),
                    decoration: new BoxDecoration(
                      border: Border.all(
                        color: AppColors.APP_COLOR_WHITE,
                      ),
                      borderRadius: BorderRadius.all(Radius.circular(5)),
                      color: AppColors.halfWhite,
                    ),
                    height: 30,
                    width: 50,
                    child: CountryCodePicker(
                      showFlagDialog: true,
                      onChanged: widget.countryCodePicker,
                      // Initial selection and favorite can be one of code ('IT') OR dial_code('+39')
                      initialSelection:
                          widget.initalSelection ?? initalSelection,
                      favorite: ['+91', 'IN'],
                      // optional. Shows only country name and flag
                      showCountryOnly: false,
                      // optional. Shows only country name and flag when popup is closed.
                      showOnlyCountryWhenClosed: false,
                      showFlag: false,
                      // optional. aligns the flag and the Text left
                      alignLeft: false,
                      textStyle: TextStyle(
                          fontFamily: FontFamily.Gilroy,
                          fontSize: FontSize.FONT_13,
                          fontWeight: FontWeight.bold),
                    ),
                  ),
                  hintText: AppStrings.MOBILE,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.themeColor != null
                          ? widget.themeColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (val.length == 0) {
                    return AppStrings.MOBILE_CANNOT_BE_EMPTY;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.phone,
                style: new TextStyle(
                  fontSize: FontSize.FONT_15,
                  letterSpacing: 2.0,
                  fontFamily: FontFamily.Poppins,
                ),
              ),
              SizedBox(
                height: 10,
              ),
              Button.mini(
                btnColor: widget.themeColor != null
                    ? widget.themeColor
                    : AppColors.primaryColor,
                onPressed: () {
                  widget.phoneNumberSignInFunction();
                },
                buttonType: ButtonType.custom,
                btnText: AppStrings.NEXT,
                btnTextColor: AppColors.APP_COLOR_WHITE,
              ),
              SizedBox(
                height: 20,
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget get _emailWidget {
    return Container(
      child: Container(
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height * 0.9,
        padding: EdgeInsets.symmetric(horizontal: 20.0),
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.only(
            topLeft: Radius.circular(5.0),
            topRight: Radius.circular(5.0),
          ),
        ),
        child: Form(
          key: widget.globalKey,
          child: Column(
            children: [
              Container(
                padding: EdgeInsets.all(20),
                child: Text(
                  AppStrings.SIGN_UP,
                  style: TextStyle(
                    fontFamily: FontFamily.Gilroy,
                    fontSize: FontSize.FONT_20,
                    color: widget.themeColor != null
                        ? widget.themeColor
                        : AppColors.primaryColor,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: [
                  Container(
                    width: MediaQuery.of(context).size.width * 0.42,
                    child: TextFormField(
                      controller: widget.firstNameController,
                      decoration: new InputDecoration(
                        hintText: AppStrings.FIRST_NAME,
                        hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                        fillColor: AppColors.appWhite,

                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: widget.themeColor != null
                                ? widget.themeColor
                                : AppColors.primaryColor,
                          ),
                        ),
                        enabledBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: AppColors.halfWhite,
                            width: 2.0,
                          ),
                        ),

                        //fillColor: Colors.green
                      ),
                      validator: (val) {
                        if (val.length == 0) {
                          return AppStrings.FIRST_NAME_CANNOT_BE_EMPTY;
                        } else {
                          return null;
                        }
                      },
                      keyboardType: TextInputType.name,
                      style: new TextStyle(
                        fontFamily: FontFamily.Poppins,
                      ),
                    ),
                  ),
                  SizedBox(
                    width: 10,
                  ),
                  Container(
                    width: MediaQuery.of(context).size.width * 0.42,
                    child: TextFormField(
                      controller: widget.lastNameController,
                      decoration: new InputDecoration(
                        hintText: AppStrings.LAST_NAME,
                        hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                        fillColor: AppColors.appWhite,

                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: widget.themeColor != null
                                ? widget.themeColor
                                : AppColors.primaryColor,
                          ),
                        ),
                        enabledBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(7.0),
                          borderSide: BorderSide(
                            color: AppColors.halfWhite,
                            width: 2.0,
                          ),
                        ),

                        //fillColor: Colors.green
                      ),
                      validator: (val) {
                        if (val.length == 0) {
                          return AppStrings.LAST_NAME_CANNOT_BE_EMPTY;
                        } else {
                          return null;
                        }
                      },
                      keyboardType: TextInputType.name,
                      style: new TextStyle(
                        fontFamily: FontFamily.Poppins,
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 10,
              ),
              TextFormField(
                controller: widget.emailController,
                decoration: new InputDecoration(
                  hintText: AppStrings.EMAIL,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.themeColor != null
                          ? widget.themeColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (!SystemUtils.hasMatchEmail(val)) {
                    return AppStrings.INVALID_EMAIL;
                  }
                  if (val.length == 0) {
                    return AppStrings.EMAIL_CANNOT_BE_EMPTY;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.emailAddress,
                style: new TextStyle(
                  fontFamily: FontFamily.Poppins,
                ),
              ),
              SizedBox(
                height: 10,
              ),
              TextFormField(
                controller: widget.passwordController,
                obscureText: !_obscureText,
                decoration: new InputDecoration(
                  suffixIcon: GestureDetector(
                    onTap: () {
                      _toggle();
                    },
                    child: Icon(
                      _passwordVisible
                          ? Icons.visibility
                          : Icons.visibility_off,
                      color: !_passwordVisible
                          ? AppColors.halfWhite
                          : AppColors.textColor,
                    ),
                  ),
                  hintText: AppStrings.PASSWORD,
                  hintStyle: TextStyle(fontFamily: FontFamily.Gilroy),
                  fillColor: AppColors.appWhite,
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: widget.themeColor != null
                          ? widget.themeColor
                          : AppColors.primaryColor,
                    ),
                  ),
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(7.0),
                    borderSide: BorderSide(
                      color: AppColors.halfWhite,
                      width: 2.0,
                    ),
                  ),
                ),
                validator: (val) {
                  if (val.length == 0) {
                    return AppStrings.PASSWORD_CANNOT_BE_EMPTY;
                  } else if (val.length < 6) {
                    return AppStrings.PASSWORD_TOO_SHORT;
                  } else {
                    return null;
                  }
                },
                keyboardType: TextInputType.text,
                style: new TextStyle(
                  fontFamily: FontFamily.Gilroy,
                ),
              ),
              SizedBox(
                height: 20,
              ),
              Button.mini(
                btnColor: widget.themeColor != null
                    ? widget.themeColor
                    : AppColors.primaryColor,
                onPressed: () {
                  widget.emailPasswordSignInFunction();
                },
                buttonType: ButtonType.custom,
                btnText: AppStrings.SIGN_UP,
                btnTextColor: AppColors.APP_COLOR_WHITE,
              ),
              SizedBox(
                height: 20,
              ),
              Row(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Container(
                    child: Text(
                      AppStrings.ALREADY_HAVE_AN_ACCOUNT,
                      style: TextStyle(
                          fontFamily: FontFamily.Gilroy,
                          fontSize: FontSize.FONT_13,
                          color: AppColors.dotColor,
                          fontWeight: FontWeight.normal),
                      textAlign: TextAlign.center,
                    ),
                  ),
                  InkWell(
                    onTap: () {
                      widget.signInFunction();
                    },
                    child: Container(
                      child: Text(
                        AppStrings.SIGN_IN,
                        style: TextStyle(
                            fontFamily: FontFamily.Gilroy,
                            fontSize: 15,
                            color: widget.themeColor != null
                                ? widget.themeColor
                                : AppColors.primaryColor,
                            fontWeight: FontWeight.bold),
                        textAlign: TextAlign.center,
                      ),
                    ),
                  ),
                ],
              ),
              SizedBox(
                height: 20,
              ),
            ],
          ),
        ),
      ),
    );
  }
}
