/*
    Refer License for Terms & Conditions
 */
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/AppStrings.dart';
import 'package:bui_component/MenuComponent.dart';
import 'package:cyclop/cyclop.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

// ignore: must_be_immutable
class ThemeColorComponent extends StatefulWidget {
  final Function() onMenuPressed;
  final Function() onBackPressed;
  Color backgroundColor;
  Color secondaryColor;

  ThemeColorComponent({
    Key key,
    this.onMenuPressed,
    this.backgroundColor,
    this.onBackPressed,
    this.secondaryColor,
  }) : super(key: key);

  @override
  ThemeColorComponentState createState() => ThemeColorComponentState();
}

class ThemeColorComponentState extends State<ThemeColorComponent> {
  Color appbarColor = Colors.blueGrey;

  Color backgroundColor;
  Color secondaryBackgroundColor;

  Set<Color> swatches = Colors.primaries.map((e) => Color(e.value)).toSet();
  Set<Color> secondarySwatches =
      Colors.primaries.map((e) => Color(e.value)).toSet();
  SharedPreferences prefs;

  @override
  void initState() {
    super.initState();
    backgroundColor = widget.backgroundColor;
    secondaryBackgroundColor = widget.secondaryColor;

    getStringValuesSF();
    getSecondaryColor();
  }

  getSecondaryColor() async {
    prefs = await SharedPreferences.getInstance();
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);

      setState(() {
        secondaryBackgroundColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColor;
    } else {
      setState(() {
        secondaryBackgroundColor = widget.secondaryColor;
        this.mounted;
      });
    }
  }

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    //Return int
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int primaryColorCode = int.parse(stringValue);
      setState(() {
        backgroundColor = Color(primaryColorCode);

        this.mounted;
      });

      return backgroundColor;
    } else {
      setState(() {
        backgroundColor = widget.backgroundColor;

        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: AppColors.APP_COLOR_WHITE,
      body: SafeArea(
        child: Center(
          child: Column(
            children: <Widget>[
              Container(
                height: 60,
                color: backgroundColor != null
                    ? backgroundColor
                    : AppColors.APP_COLOR_PRIMARY,
                child: Row(
                  children: <Widget>[
                    ClipRRect(
                      borderRadius: BorderRadius.all(Radius.circular(32.0)),
                      child: InkWell(
                        splashColor: AppColors.errorWarning,
                        focusColor: AppColors.APP_COLOR_WHITE,
                        onTap: () {
                          getStringValuesSF();
                          //widget.onMenuPressed();
                          widget.onBackPressed();
                        },
                        child: Padding(
                          padding: const EdgeInsets.all(12.0),
                          child: Icon(
                            Icons.arrow_back,
                            color: Colors.white,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
              Expanded(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: <Widget>[
                    _primarColor,
                    SizedBox(
                      height: 20,
                    ),
                    _secondaryColor,
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget get _primarColor {
    final textTheme = Theme.of(context).textTheme;

    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        Center(
          child: Text(
            "Primary",
            style: textTheme.headline6.copyWith(
                color: AppColors.APP_COLOR_BLACK, fontWeight: FontWeight.bold),
          ),
        ),
        Container(
          child: ColorButton(
            darkMode: false,
            key: Key('c1'),
            color: backgroundColor,
            swatches: swatches,
            onColorChanged: (value) {
              setState(() {
                backgroundColor = value;

                MenuComponentState.color = [
                  backgroundColor,
                  AppColors.appWhite
                ];
                setPrefs(value);
                this.mounted;
              });
            },
            onSwatchesChanged: (newSwatches) {
              setState(() {
                swatches = newSwatches;
              });
            },
          ),
        ),
      ],
    );
  }

  Widget get _secondaryColor {
    final textTheme = Theme.of(context).textTheme;

    final bodyTextColor =
        ThemeData.estimateBrightnessForColor(backgroundColor) == Brightness.dark
            ? Colors.white70
            : Colors.black87;

    return Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        Container(
          padding: EdgeInsets.all(1),
          child: Text(
            "Secondary",
            style: textTheme.headline6
                .copyWith(color: Colors.black ?? bodyTextColor),
          ),
        ),
        Center(
          child: ColorButton(
            darkMode: false,
              key: Key('c2'),
            color: secondaryBackgroundColor,
            swatches: secondarySwatches,
            onColorChanged: (value) {
              setState(() {
                secondaryBackgroundColor = value;
                setSecondaryPrefs(value);
              });
            },
            onSwatchesChanged: (newSwatches) =>
                setState(() => secondarySwatches = newSwatches),
          ),
        ),
      ],
    );
  }

  void setPrefs(Color value) async {
    var parts = value.toString().split('(');
    var date = parts.sublist(1).join('(').trim();
    List<String> c = date.split("");
    c.removeLast();
    String colorCode = c.join();

    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(AppStrings.PRIMARY_THEME_COLOR, colorCode.toString());
  }

  void setSecondaryPrefs(Color value) async {
    var parts = value.toString().split('(');
    var date = parts.sublist(1).join('(').trim();
    List<String> c = date.split("");
    c.removeLast();
    String colorCode = c.join();

    SharedPreferences prefs = await SharedPreferences.getInstance();
    prefs.setString(AppStrings.SECONDARY_THEME_COLOR, colorCode.toString());
  }
}
